/*
 * PeRdr - PE file disassembler
 * Copyright (C) 2006 Frediano Ziglio
 * -----
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * -----
 *
 * INFORMATION
 *   www: http://freddy77.tripod.com/perdr/
 *     e-mail: freddy77@angelfire.com
 *     */
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "apiarguments.hpp"
#include "file.hpp"
#include <cstring>
#include <stdexcept>

#ifdef WIN32

#undef ULONG
#undef DWORD
#undef INT
#undef UINT
#undef LONG
#include <windows.h>

#endif


extern const char* argv0;       // defined in perdr.cpp

bool ApiArguments::Populate()
{
  bool ok = false;
#define WINAPIFN "winapifn.lst"

#ifdef PKGDATADIR
  if (!ok)
    ok = Populate(PKGDATADIR "/" WINAPIFN);
#endif

  if (!ok)
  {
    std::string winapi;
    std::string::size_type pos;
#ifdef WIN32
    char buf[MAX_PATH];
    GetModuleFileName(NULL, buf, sizeof(buf));
    winapi = buf;
    pos = winapi.find_last_of("/\\:");
#else
    winapi = argv0;
    pos = winapi.find_last_of("/");
#endif
    if (pos == std::string::npos)
      pos = 0;
    else
      ++pos;
    winapi.replace(pos, std::string::npos, WINAPIFN);
    ok = Populate(winapi.c_str());
  }

  if (!ok)
    ok = Populate(WINAPIFN);

  return ok | Populate("customfn.lst");
}


bool ApiArguments::Populate(const char* name)
{
  try
  {
    CFile file(name, "r");
    for (;;)
    {
      char* line = file.ReadString('\n');
      char* func = strtok(line,",");
      char* name = strchr(func,'=');
      if (name)
      {
        *name++ = '\0';
        aliases.insert(name,strtoul(func,0,16));
        func = name;
      }
      char* arg;
      TFnArgs args;
      while ((arg = strtok(0,",")))
      {
        char* ob;
        if ((ob = strrchr(arg,'(')) != NULL)
        {
          // found function parameter, join its argument list
          char* cb = strrchr(arg,')');
          if (cb > ob) // type (*)(type)
            args.push_back(arg);
          else         // type (*)(type
          {
            std::string a(arg);
            do
            {
              arg = strtok(0,",");
              if (arg)
              {
                a += ",";
                a += arg;
              }
            } while (arg && !strchr(arg,')'));
            args.push_back(a);
          }
        }
        else
          args.push_back(arg);
      }
      if (!args.empty())
        fn[func] = args;
      delete [] line;
    }
  }
  catch (const std::runtime_error& e)
  {
    if (strcmp(e.what(),"Unable to open file") == 0)
      return false;
  }
  return true;
}

const TFnArgs* ApiArguments::find(const std::string& func) const
{
  // strip the DLL name, if present
  std::string name;
  unsigned dot = func.find(".");
  if (dot == std::string::npos)
    name = func;
  else
    name = func.substr(dot + 1);
  TApiFn::const_iterator i = fn.find(name);
  if (i == fn.end())
    return 0;
  return &(i->second);
}

const TFnArgs* ApiArguments::find(vma_t address) const
{
  char str[12];
  sprintf(str,"%08X",address);
  return find(str);
}

