/*
 * PeRdr - PE file disassembler
 * Copyright (C) 1999-2006 Frediano Ziglio
 * -----
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * -----
 *
 * INFORMATION
 *   www: http://freddy77.tripod.com/perdr/
 *     e-mail: freddy77@angelfire.com
 *     */
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include <algorithm>
#include "apicache.h"
#include "apiexp.h"

using namespace std;

#include <map>
#include "utils.hpp"
#include "file.hpp"
#include "pefile.hpp"

typedef map<int, std::string> TDllNames;
typedef map<std::string,TDllNames> TDllsNames;
static TDllsNames dllsNames;

static void DllNamesAddImport(uint32_t address,uint32_t hint,const char* dll_name,const char* func_name,void* param)
{
  TDllNames *dllNames = (TDllNames*)param;
  if (address != 0 && func_name != NULL)
    (*dllNames)[hint] = func_name;
}

static bool ExternFullApi(std::string& out,const char* dllName,int ordinal)
{
  TDllsNames::iterator dll = dllsNames.find(dllName);
  if (dll == dllsNames.end())
  {
    // !!! capitalize dllName
    // insert an empty dll
    dllsNames.insert(TDllsNames::value_type(dllName,TDllNames()));
    dll = dllsNames.find(dllName);

    // search DLL/OCX
    std::string filename = FindDll(dllName);
    
    if (filename == "")
      return false;

    CFile file(filename.c_str(), "rb");

    PeFile peFile(file);

    // parse exports
    const PE_IMAGE_DATA_DIRECTORY& export_ =
      peFile.GetDataDirectory(IMAGE_DIRECTORY_ENTRY_EXPORT);
    if ( export_.VirtualAddress != 0 && export_.Size != 0 )
    {
      TDllNames *dllNames = &(*dll).second;
      peFile.ParseExport(DllNamesAddImport,(void*)dllNames);
    }
  }
  
  TDllNames::iterator name = (*dll).second.find(ordinal);
  if (name == (*dll).second.end())
    return false;
  out = (*name).second;
  return true;
}

bool ApiCache::GetFullApiName(std::string& out,const char* dllName,int ordinal)
{
  // find dll
  // FIXME do a best search, dll should be ordered
  const DllFunctions *dll;
  for(dll = cachedDlls; ; )
  {
    if (stricmp(dll->name,dllName)==0)
      break;
    ++dll;
    if (dll == (cachedDlls + numCachedDlls) )
      return ExternFullApi(out, dllName, ordinal);
  }

  // find function
  const int *pend = dll->ords + dll->count;
  const int *p = lower_bound(dll->ords,pend,ordinal);
  if ( pend == p || *p != ordinal)
    return false;
  out = dll->names[p-dll->ords];
  return true;
}

