/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "byteinfo.h"

ByteInfo& ByteInfos::operator[](vma_t address)
{
  TInfos::const_iterator i = map.find(address/unsigned(PageSize));
  if ( i != map.end() )
    return (*(*i).second)[address%unsigned(PageSize)];

  map[address/unsigned(PageSize)] = freddy77::hp_auto_ptr<TPage>(new TPage());
  return (*map[address/unsigned(PageSize)])[address%unsigned(PageSize)];
}

const ByteInfo ByteInfos::emptyInfo;

/*
ByteInfos::TSetInfoResult ByteInfos::SetInfo(vma_t address,unsigned len,ByteInfo::TType type,ByteInfo::TPriority priority)
{
  TInfos::iterator i = map.find(address/unsigned(PageSize));
  unsigned partial = address%unsigned(PageSize);
  if ( i == map.end() )  
      i = map.insert( TInfos::value_type(address/unsigned(PageSize), freddy77::hp_auto_ptr<TPage>(new TPage()) ) ).first;
}
*/

const ByteInfo& ByteInfos::operator[](vma_t address) const
{
  TInfos::const_iterator i = map.find(address/unsigned(PageSize));
  if ( i != map.end() )
  return (*(*i).second)[address%unsigned(PageSize)];

  // never alter map
  return emptyInfo;
}

uchar ByteInfos::GetIstrLen(vma_t address) const
{
  TInfos::const_iterator i = map.find(address/unsigned(PageSize));
  if ( i == map.end() ) return 0;
  return (*(*i).second)[address%unsigned(PageSize)].len;
}

void ByteInfos::SetLen(vma_t address,unsigned len)
{
  while(len>32)
  {
    ByteInfo &info = (*this)[address];
    info.len = 32;
    info.SetContinue();
    len -= 32;
    address += 32;
  }
  ByteInfo &info = (*this)[address];
  info.len = (uchar)len;
}

unsigned ByteInfos::GetLen(vma_t address) const
{
  unsigned len;
  const ByteInfo *info = &((*this)[address]);
  len = info->len;
  while(info->Continue())
  {
    info = &((*this)[address+len]);
    len += info->len;
  }
  return len;
}

// il codice al dato indirizzo e' occupato da un'altra istruzione ?
bool ByteInfos::IsOccupied(vma_t address) const
{
  // !!! 32 = max istr length
  const int maxIstrSize = 32;
  TInfos::const_iterator i = map.find(address/unsigned(PageSize));
  unsigned partial = address%unsigned(PageSize);

  // caso tutti su una pagina
  if (partial>=(maxIstrSize-1))
  {
    if ( i == map.end())
      return false;
    TPage &page = *(*i).second;
    for(int n=0;n<maxIstrSize;++n,--partial)
      if (page[partial].len > n)
        return true;
    return false;
  }

  // sfortuna, 2 pagine
  for(int n=0;n<maxIstrSize;++n)
    if (GetIstrLen(address-n) > n)
      return true;
  return false;
}

bool ByteInfos::IsOccupied(vma_t address,int bytes) const
{
  _PRG_ASSERT(bytes>0);
  // !!! 32 = max istr length
  const int maxIstrSize = 32u;
  address += --bytes;
  TInfos::const_iterator i = map.find(address/unsigned(PageSize));
  unsigned int partial = address%unsigned(PageSize);

  // caso tutti su una pagina
  if (partial>=(unsigned int)(maxIstrSize-1+bytes))
  {
    if ( i == map.end())
      return false;
    TPage &page = *(*i).second;
    int n = -bytes;
    for(;n<0;++n,--partial)
      if (page[partial].len > 0)
        return true;
    for(;n<maxIstrSize;++n,--partial)
      if (page[partial].len > n)
        return true;
    return false;
  }

  // sfortuna, 2 pagine
  int n = -bytes;
  address -= bytes;
  for(;n<0;++n)
    if (GetIstrLen(address-n) > 0)
      return true;
  for(;n<maxIstrSize;++n)
    if (GetIstrLen(address-n) > n)
      return true;
  return false;
}

