/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#ifndef FILE_DATAREADER_H
#define FILE_DATAREADER_H

class DataReader
{
private:
  uint8_t ReadU8Slow();
  uint16_t ReadU16LESlow();
  uint32_t ReadU32LESlow();
protected:
  const unsigned char* ptr;
  const unsigned char* end;  
  /** read next continue block. Trow exception if impossible  */
  virtual void ReadNext()=0;
public:
  inline DataReader(const unsigned char* ptr,const unsigned char* end);
  virtual ~DataReader() {};
  inline uint8_t ReadU8();
  inline uint16_t ReadU16LE();
  inline uint32_t ReadU32LE();
  virtual void UnReadU8()=0;
  virtual vma_t Tell()=0;
};

inline DataReader::DataReader(const unsigned char* _ptr,const unsigned char* _end):
  ptr(_ptr), end(_end)
{
}

inline uint8_t DataReader::ReadU8()
{
  if (ptr < end)
    return *ptr++;
  return ReadU8Slow();
}

inline uint16_t DataReader::ReadU16LE()
{
  if (ptr + 1 < end) 
  {
    uint16_t res = ptr[0] + (uint16_t(ptr[1]) << 8);
    ptr += 2;
    return res;
  }
  return ReadU16LESlow();
}

inline uint32_t DataReader::ReadU32LE()
{
  if (ptr + 3 < end)
  {
    uint32_t res = ptr[0] + (uint32_t(ptr[1]) << 8) + (uint32_t(ptr[2]) << 16) 
      + (uint32_t(ptr[3]) << 24);
    ptr += 4;
    return res;
  }
  return ReadU32LESlow();
}

#endif // FILE_DATAREADER_H
