/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#ifndef FILE_MODULE_H
#define FILE_MODULE_H

#include "rva.h"
#include "file.hpp"
#include "utils/range.hpp"
#include "relocationinfo.h"
#include "symbols.h"
#include "datareader.h"

// gestisce il file come se fosse caricato in memoria
// throw exception
class ObjectModule
{
public:
  /* the X fix a severe bug of Visual C++ 6 SP5 do not use X version !!! */
  class XDataReader;
  class Section: public range<vma_t>
  {
  friend class ObjectModule;
  friend class XDataReader;
  public:
    Section(): range<vma_t>(0,0),flag(0),data(NULL),data_end(NULL),next(NULL) {};
    ~Section();
    // they was bool, but this cause some warning
    unsigned IsCode() const { return flag & fCode; }
    unsigned IsData() const { return flag & fData; }
    void SetIsCode() { flag |= fCode; }
    void SetIsData() { flag |= fData; }
  private:
    enum { fCode = 1, fData = 2, fWritable = 4 };
    unsigned flag;
    unsigned char* data;
    unsigned char* data_end;
    Section* next; /* for read beyond */
  };
  typedef std::vector<Section> TSections;
  /** Class for read data on Module */
  class XDataReader: public ::DataReader {
    friend class ObjectModule;
    const Section* sec;
    XDataReader(const Section* sec,const unsigned char* ptr);
    void ReadNext();
  public:
    void UnReadU8();
    vma_t Tell();
  };
  typedef XDataReader DataReader;
  XDataReader GetDataReader(vma_t address);

  ObjectModule(CFile& file,const RVAFileTranslator& rva,uint32_t imageBase,bool hasRelocation,const RelocationInfos& relocationInfos, const Symbols& symbols, const Symbols& exportSymbols, vma_t entryPoint);
  ~ObjectModule();
  bool IsValid(vma_t address);
//  vma_t GetNextValid();

  // aggiunti per "compatibilita'" bfd
  bool HasRelocation() const
  { return hasRelocation; }
  const RelocationInfos& GetRelocationInfos() const
  { return relocationInfos; }
  const Symbols& GetSymbols() const
  { return symbols; }
  const Symbols& GetExportSymbols() const
  { return exportSymbols; }
  vma_t GetEntryPoint() const 
  { return entryPoint; }

  // section access
  // se non trovata nessuna sezione ritorna GetErrorSection()
  const Section* GetSection(vma_t address) const;
  // sezione usata per ritorno in caso di sezione non trovata
  static const Section* GetErrorSection() { return &errorSection; }

  //
  const TSections& GetSections() const
  { return sections; }
private:
  TSections sections;
//  Section* currSection;
  // not implemented
  void operator=(const ObjectModule&);
  ObjectModule(const ObjectModule&);
  static const Section errorSection;

  bool hasRelocation;
  RelocationInfos relocationInfos;

  Symbols symbols, exportSymbols;
  vma_t entryPoint;
};
#endif // FILE_MODULE_H
