/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "param.h"

const unsigned Param::memTypeSizes[Param::memTypeCount] =
{
#define DEFINE_MEMTYPE(name,size) size,
#define DEFINE_MEMTYPEI(name,size) size,
#include "memtype.h"
#undef DEFINE_MEMTYPE
#undef DEFINE_MEMTYPEI
};

extern void ReadInstruction(const char* s);
extern void ReadParam(Param& prm,const char* s);

Param::Param(enum TCRegistry ,reg_t reg):
  relocation(NULL)
{
  type = t_registry;
  mem_reg1 = reg;
  mem_reg2 = null_reg;
  literal_size = 0;
  // size = ?? from reg
}

Param::Param(enum TCLiteral ,int _literal,unsigned char _size):
  relocation(NULL)
{
  _PRG_ASSERT(_size != 0);
  type = t_literal;
  mem_reg1 = null_reg;
  mem_reg2 = null_reg;
  literal = _literal;
  literal_size = _size;
}

Param::Param(enum TCFarLiteral ,int _literal,unsigned char _size,int segment):
  relocation(NULL)
{
  _PRG_ASSERT(_size != 0);
  type = t_farliteral;
  mem_reg1 = static_cast<reg_t>(segment); // !!!
  mem_reg2 = null_reg;
  literal = _literal;
  literal_size = _size;
}

Param::Param(enum TCMemory ,int address,unsigned char address_size,MemoryTypes _memType):
  relocation(NULL)
{
  _PRG_ASSERT(address_size != 0 || address == 0);
  type = t_memory;
  mem_reg1 = null_reg;
  mem_reg2 = null_reg;
  literal = address;
  literal_size = address_size;
  memType = _memType;
}

Param::Param(enum TCMemory ,reg_t reg,int address,unsigned char address_size,MemoryTypes _memType):
  relocation(NULL)
{
  _PRG_ASSERT(address_size != 0 || address == 0);
  type = t_memory;
  mem_reg1 = reg;
  mem_reg2 = null_reg;
  literal = address;
  literal_size = address_size;
  memType = _memType;
}

Param::Param(enum TCMemory ,reg_t reg1,reg_t reg2,int address,unsigned char address_size,unsigned char multiplier,MemoryTypes _memType):
  relocation(NULL)
{
  _PRG_ASSERT(address_size != 0 || address == 0);
  type = t_memory;
  mem_reg1 = reg1;
  mem_reg2 = reg2;
  literal = address;
  literal_size = address_size;
  factor = multiplier;
  memType = _memType;
}

#ifdef DEBUG
void Param::InitInvalid()
{
  type = (enum Types) (t_farliteral + 1);
  literal_size = 0;
}

void Param::CheckValid()
{
  _PRG_ASSERT(type >= t_registry && type <= t_farliteral);
  if (type == t_literal || type == t_farliteral)
    _PRG_ASSERT(literal_size != 0);
  if (type == t_memory)
    _PRG_ASSERT(literal_size != 0 || literal == 0);
  _PRG_ASSERT(literal_size <= 4 && literal_size != 3);
}
#endif

unsigned char Param::GetSize() const
{
  switch (type)
  {
  default:
    _PRG_ASSERT(0);
  case t_memory:
    return GetMemSize();
  case t_literal:
  case t_farliteral:
    return GetLiteralSize();
  case t_registry:
    return RegistryInfo::GetInfo(mem_reg1)->GetByteSize();
  }
}

