/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include <getopt.h>
#include "perdroptions.h"

PeRdrOption options = {
  false, // showBytes
  false, // disableP3
  false, // disable3DNow
  false, // disableEnh3DNow
  false, // disableAthlon
  false, // showVersion
  false, // showHelp
  false, // showPriority
  false, // rawDumping
  false, // extractIconRes
  false, // showHeaders
  false, // fullHeaders
  false, // showImports
  false, // showRelocations
  false, // showExports
  false, // showResources
  false, // rawResources
  false, // showCode
  false, // showcRef
  false, // showURef
  false, // showCRef
  false, // entryIsNotCode
};

static struct option long_options[] = {
  {"help",                  0, 0,   1},
  {"version",               0, 0, 'v'},
  {"bytes",                 0, 0, 'b'},
  {"show-priority",         0, 0, 'p'},
  {"raw-dump",              0, 0,   2},
  {"extract-icon-resource", 0, 0,   3},
  {"disable-3dnow",         0, 0,   4},
  {"disable-enh3dnow",      0, 0,   5},
  {"disable-pentium3",      0, 0,   6},
  {"disable-athlon",        0, 0,   7},
  {"show-headers",          0, 0, 'h'},
  {"full-headers",          0, 0, 'H'},
  {"show-imports",          0, 0, 'i'},
  {"show-relocations",      0, 0,   8},
  {"show-exports",          0, 0, 'e'},
  {"show-resources",        0, 0, 'r'},
  {"raw-resources",         0, 0, 'R'},
  {"show-code",             0, 0, 'c'},
  {"show-cref",             0, 0,   9},
  {"show-Uref",             0, 0,  10},
  {"show-Cref",             0, 0,  11},
  {"show-refs",             0, 0,  12},
  {"addr-start",            1, 0,  13},
  {"addr-end",              1, 0,  14},
  {"no-entry-code",         0, 0,  15},
  {0, 0, 0, 0}
};

int ReadOptions(int argc,char** argv)
{
  int c, option_index, res;

  res = argc - 1; 
  while ((c = getopt_long (argc, argv, "vbpHiherRc", long_options, &option_index)) != -1) {
    switch (c) {
      case   1: options.showHelp        = true; break;
      case 'v': options.showVersion     = true; break;
      case 'b': options.showBytes       = true; break;
      case 'p': options.showPriority    = true; break;
      case   2: options.rawDumping      = true; break;
      case   3: options.extractIconRes  = true; break;
      case   4: options.disable3DNow    =
                options.disableEnh3DNow = true; break;
      case   5: options.disableEnh3DNow = true; break;
      case   6: options.disableP3       = true; break;
      case   7: options.disableAthlon   = true; break;
      case 'h': options.showHeaders     = true; break;
      case 'H': options.fullHeaders     = true; break;
      case 'i': options.showImports     = true; break;
      case   8: options.showRelocations = true; break;
      case 'e': options.showExports     = true; break;
      case 'r': options.showResources   = true; break;
      case 'R': options.rawResources    = true; break;
      case 'c': options.showCode        = true; break;
      case   9: options.showcRef        = true; break;
      case  10: options.showURef        = true; break;
      case  11: options.showCRef        = true; break;
      case  12: options.showcRef        =
                options.showURef        =
                options.showCRef        = true; break;
      case  13: options.addrStart = strtoul(optarg, NULL, 0); break;
      case  14: options.addrEnd   = strtoul(optarg, NULL, 0); break;
      case  15: options.entryIsNotCode  = true; break;
      default: res = optind;
    }
  }

  if (!options.showHeaders && !options.showImports && !options.showRelocations 
      && !options.showExports && !options.showCode
      && !options.showResources)
  {
    options.showHeaders = true;
    options.showImports = true;
    options.showRelocations = true;
    options.showExports = true;
    options.showCode = true;
    options.showResources = true;
  }

  return res;
}
