/*
PeRdr - PE file disassembler
Copyright (C) 1999-2002 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#ifndef FILE_REGISTERS_H
#define FILE_REGISTERS_H

#define DEFINE_REG(size,type,name,string,complete,mask) \
  __reg_##name,
typedef enum _reg_t {
  null_reg = -1,
#include "x86regs.h"
  __reg_Count
} reg_t;
#undef DEFINE_REG

typedef struct reg_info
{
  enum reg_size { bit = 0,byte = 1,word = 2,dword = 4,packetI = 8,packetF = 16,floating = 10};
  enum reg_size size;
  enum reg_type { flag,byte_low_low,byte_low_high,word_low,general,segment,special };
  enum reg_type type;
  const char* name;
  int completeReg;      // valido per parziali, uguale a reg se non parziale
  unsigned partialMask; // !!! quanti bits ?
  unsigned char GetByteSize() const
  { return (unsigned char)size; }
} reg_info;

#ifdef DEBUG
#include <stdexcept>
#endif

class RegistryInfo
{
public:
  static const reg_info* GetInfo(const char* name);
  static const reg_info* GetInfo(reg_t i);
  static reg_t GetIndex(const char* name);
#ifdef DEBUG
  static reg_t GetRegSafe(const char* name)
  {
    reg_t reg = GetIndex(name);
    if (reg == null_reg)
      throw std::runtime_error(name);
    return reg;
  }
#endif
};

#define REG(s) __reg_##s

#endif //FILE_REGISTERS_H
