/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include <algorithm>
#include <iostream>
#include "symbols.h"
#include "apicache.h"

using namespace std;

// !!! sebbene il costruttore sia vuoto e' bene non renderlo inline
Symbols::Symbols()
{
}


pair<Symbols::iterator,Symbols::iterator> Symbols::operator[](vma_t address) const
{
  iterator begin = symbols.lower_bound(Symbol(address,""));
  iterator end   = begin;
  
  // usually do one loop, so do not use strange method
  for(; end != symbols.end() ; ++end)
    if ( (*end).address != address )
      break;
  _PRG_ASSERT( begin == symbols.end() || (*begin).address >= address );
  _DEBUG_(iterator tmp = begin);
  _PRG_ASSERT( tmp == symbols.begin() || (*--tmp).address < address );
  _PRG_ASSERT( end == symbols.end() || begin == end || (*end).address > address );
  if( end == symbols.end() || begin == end || (*end).address > address ) ;
    else { cerr << "Address=" << address << " end Address=" << (*end).address << " end Name=" << (*end).name << endl; _PRG_ASSERT(0); }
  _DEBUG_( tmp = end );
  _PRG_ASSERT( tmp == symbols.begin() || begin == end || (*--tmp).address == address );
  return pair<iterator,iterator>(begin,end);
}

bool Symbols::IsValid(vma_t address) const
{
  pair<iterator,iterator> apis = (*this)[address];
  if ( apis.first != apis.second )
    return true;
  return false;
}

void Symbols::insert(const std::string& name,vma_t address)
{
  symbols.insert(Symbol(address,name));
}

vma_t Symbols::GetNextValid(vma_t address) const
{
  iterator i = symbols.lower_bound(Symbol(address+1,""));
  if ( i == symbols.end() )
    return 0;
  _PRG_ASSERT( (*i).address > address );
  return (*i).address;
}

void SymbolsAddImport(uint32_t address,uint32_t hint,const char* dll_name,const char* func_name,void* param)
{
  if (address != 0)
  {
    SymbolsAddImportParam *p = (SymbolsAddImportParam*)param;
    char s[10]; // !!! constant
    std::string name(dll_name);
    name += ".";
    if (func_name != NULL)
      name += func_name;
    else
    {
      // name += itoa(hint,s,10);
      sprintf(s,"%i",hint);
      name += s; //itoa(hint,s,10);
      
      std::string fullName;
      if (p->isImport && ApiCache::GetFullApiName(fullName,dll_name,hint))
      {
        name += " (";
        name += fullName;
        name += ")";
      }
    }
    p->symbols->insert(name,address+p->imageBase);
  }
}

