/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#ifndef FILE_SYMBOLS_H
#define FILE_SYMBOLS_H

#include <string>
#include <set>

/**
 * Contain all info for API call (imports and exports)
 */
class Symbols
{
private:
  /**
   * hold info (api name and address) on a single symbol
   */
  struct Symbol
  {
    vma_t address; /**< address of symbol */
    std::string name; /**< name of symbol */
    Symbol(vma_t _address,const std::string& _name):
      address(_address),name(_name)
    { }
    bool operator<(const Symbol& rhs) const
    { return address < rhs.address || (address == rhs.address && name < rhs.name); }
    bool operator==(const Symbol& rhs) const
    { return address == rhs.address && name == rhs.name; }
  };
  typedef std::set<Symbol,std::less<Symbol> > t_symbols;
  t_symbols symbols;
public:
  typedef t_symbols::const_iterator iterator;
  Symbols();
  // "" if not found
  /** Return names at a given address
   * @param address address
   * @return a pair of iterator (begin, end) with all api infos
   */
  std::pair<iterator,iterator> operator[](vma_t address) const;
  // name compatible with STL std::map
  /**
   * Add an api entry
   */
  void insert(const std::string& name,vma_t address);
  /**
   * Return next valid address where an api is located
   * @param address address to start searching for api
   */
  vma_t GetNextValid(vma_t address) const;
  /**
   * Are there an api at given address
   * @param address address to query
   */
  bool IsValid(vma_t address) const;
};

struct SymbolsAddImportParam
{
  Symbols* symbols;
  uint32_t imageBase;
  bool isImport;
};

// aggiungi una api eseguendo il parse
// il parametro param punta a una struttura di tipo ApiAddImportParam
void SymbolsAddImport(uint32_t address,uint32_t hint,const char* dll_name,const char* func_name,void* param);

#endif //FILE_SYMBOLS_H
