/*
PeRdr - PE file disassembler
Copyright (C) 1999-2003 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#ifndef FILE_X86DASM_H
#define FILE_X86DASM_H

#include "x86instr.h"
#include "datareader.h"

// dimensione di un indirizzo
extern int addr_bytes;

// abilita istruzioni PentiumIII
extern bool p3enable;
// abilita istruzioni 3DNow
extern bool k6_3DNow_enable;
// abilita istruzioni 3DNow enhanched
extern bool k6_en_3DNow_enable;
// abilita istruzioni Athlon
extern bool athlon_enable;

// !!! add class processor status like
// --UML------
//  ProcessorStatus
// -----------
//  16/32 bit
//  type flag (386/486, 3DNow!...)
// -----------
// -----------

class RelocationInfos;
class InstructionDecoder
{
public:
  InstructionDecoder(const RelocationInfos& r):
      reloc(r),reader(NULL)
  {};
  int Decode(Instruction& instruction,DataReader& reader);
protected:
  const RelocationInfos& reloc;
  DataReader* reader;
private:
  // table information
  typedef int (*DecodeProc)(InstructionDecoder*, uchar, int, Instruction&);
  // contain relocation, current address, buffer, etc
  struct InstTable {               //-- Instruction Table --
    DecodeProc rtn;                // Rtn to process byte
    int     special;               // Data to pass to rtnn
    int     instruction;           // Data to pass to rtn
  };
  static const InstTable instrTable[];
  static const InstTable exInstrTable[];
  inline void CheckTables();

  // read 1 byte and check for no relocation, return true if error
  bool get_checkc( uint32_t* offset );
  // read 2/4 bytes
  uint32_t ReadVar( int flag );
  // read 2/4 bytes and relocation info
  bool getv_rel( Param& param, int flag );

  // read memory info
  int do_sib( uchar* base, Param& param );
  int mod0( int sib_offset, uchar r_m, bool* size_known, Param& param );
  int mod1( int sib_offset, uchar r_m, Param& param );
  int mod2( int sib_offset, uchar r_m, bool* size_known, Param& param );
  int do_mod_rm( uchar mod, uchar r_m, int reg_size, 
     Param::MemoryTypes memType, bool size_needed, Param& param );

  // scan other table instruction
  int check_forward( const InstTable* inst_table, Instruction& instruction );

  int mod_reg2 ( uchar opcode, int special, int mod_reg, 
     Instruction& instruction );
#define INSTPROCDECL(proc) \
  static int proc(InstructionDecoder* _this, uchar opcode, \
      int special, Instruction& instruction)

  INSTPROCDECL(stub);
  INSTPROCDECL(unimpl);

  INSTPROCDECL(conv_byte);
  INSTPROCDECL(one_byte);
  INSTPROCDECL(two_byte);
  INSTPROCDECL(two_sbyte);
  INSTPROCDECL(three_byte);
  INSTPROCDECL(five_byte);

  INSTPROCDECL(one_a);
  INSTPROCDECL(two_a);
  INSTPROCDECL(three_a);

  INSTPROCDECL(in_out);
  INSTPROCDECL(string_byte);
  INSTPROCDECL(enter);
  INSTPROCDECL(two_bcd);

  INSTPROCDECL(disp8);
  INSTPROCDECL(disp16);

  INSTPROCDECL(wait);
  INSTPROCDECL(prefix);
  INSTPROCDECL(prefixf2);
  INSTPROCDECL(prefixf3);
  INSTPROCDECL(seg_over);

  INSTPROCDECL(opsize_over);
  INSTPROCDECL(adrsize_over);

  INSTPROCDECL(mod_reg);
  INSTPROCDECL(mod_xmm);
  INSTPROCDECL(mod_sse2);
  INSTPROCDECL(mod_reg_p3);
  INSTPROCDECL(seg_reg);
  INSTPROCDECL(group1);
  INSTPROCDECL(group2);
  INSTPROCDECL(group3);
  INSTPROCDECL(group4);
  INSTPROCDECL(group5);
  INSTPROCDECL(group6);
  INSTPROCDECL(group7);
  INSTPROCDECL(group8);
  INSTPROCDECL(group9);
  INSTPROCDECL(groupA);
  INSTPROCDECL(group15);
  INSTPROCDECL(group16);
  INSTPROCDECL(mov_special);
  INSTPROCDECL(mov_creg);

  INSTPROCDECL(esc);
  INSTPROCDECL(extra);

  // 3D Now!
  INSTPROCDECL(one_byte_k63d);
  INSTPROCDECL(k6_3dnow);
  INSTPROCDECL(prefetch_k63d);
#undef INSTPROCDECL
};

#endif
