/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "x86istr.h"
#include "x86table.h"
#include "x86dasm.h"
#include "utils/compilecheck.hpp"

#define istr__ -1
#define I(rtn,special,istruction) \
  { &InstructionDecoder::rtn, special, istr_##istruction }

const InstructionDecoder::InstTable InstructionDecoder::instrTable[] =
{
  I( mod_reg,  modregEbGb,  add    ), /* ADD     mem/reg,reg (byte) [00] */
  I( mod_reg,  modregEvGv,  add    ), /* ADD     mem/reg,reg (word) [01] */
  I( mod_reg,  modregGbEb,  add    ), /* ADD     reg,mem/reg (byte) [02] */
  I( mod_reg,  modregGvEv,  add    ), /* ADD     reg,mem/reg (word) [03] */
  I( two_byte,      regAL,  add    ), /* ADD     AL,kk              [04] */
  I( three_byte,    regAX,  add    ), /* ADD     AX,jjkk            [05] */
  I( one_byte,      regES,  push   ), /* PUSH    ES                 [06] */
  I( one_byte,      regES,  pop    ), /* POP     ES                 [07] */
  I( mod_reg,  modregEbGb,  or     ), /* OR      mem/reg,reg (byte) [08] */
  I( mod_reg,  modregEvGv,  or     ), /* OR      mem/reg,reg (word) [09] */
  I( mod_reg,  modregGbEb,  or     ), /* OR      reg,mem/reg (byte) [0A] */
  I( mod_reg,  modregGvEv,  or     ), /* OR      reg,mem/reg (word) [0B] */
  I( two_byte,      regAL,  or     ), /* OR      AL,kk              [0C] */
  I( three_byte,    regAX,  or     ), /* OR      AX,jjkk            [0D] */
  I( one_byte,      regCS,  push   ), /* PUSH    CS                 [0E] */
  I( extra,             0,  _      ), /* 2-byte opcode              [0F] */
  I( mod_reg,  modregEbGb,  adc    ), /* ADC     mem/reg,reg (byte) [10] */
  I( mod_reg,  modregEvGv,  adc    ), /* ADC     mem/reg,reg (word) [11] */
  I( mod_reg,  modregGbEb,  adc    ), /* ADC     reg,mem/reg (byte) [12] */
  I( mod_reg,  modregGvEv,  adc    ), /* ADC     reg,mem/reg (word) [13] */
  I( two_byte,      regAL,  adc    ), /* ADC     AL,kk              [14] */
  I( three_byte,    regAX,  adc    ), /* ADC     AX,jjkk            [15] */
  I( one_byte,      regSS,  push   ), /* PUSH    SS                 [16] */
  I( one_byte,      regSS,  pop    ), /* POP     SS                 [17] */
  I( mod_reg,  modregEbGb,  sbb    ), /* SBB     mem/reg,reg (byte) [18] */
  I( mod_reg,  modregEvGv,  sbb    ), /* SBB     mem/reg,reg (word) [19] */
  I( mod_reg,  modregGbEb,  sbb    ), /* SBB     reg,mem/reg (byte) [1A] */
  I( mod_reg,  modregGvEv,  sbb    ), /* SBB     reg,mem/reg (word) [1B] */
  I( two_byte,      regAL,  sbb    ), /* SBB     AL,kk              [1C] */
  I( three_byte,    regAX,  sbb    ), /* SBB     AX,jjkk            [1D] */
  I( one_byte,      regDS,  push   ), /* PUSH    DS                 [1E] */
  I( one_byte,      regDS,  pop    ), /* POP     DS                 [1F] */
  I( mod_reg,  modregEbGb,  and    ), /* AND     mem/reg,reg (byte) [20] */
  I( mod_reg,  modregEvGv,  and    ), /* AND     mem/reg,reg (word) [21] */
  I( mod_reg,  modregGbEb,  and    ), /* AND     reg,mem/reg (byte) [22] */
  I( mod_reg,  modregGvEv,  and    ), /* AND     reg,mem/reg (word) [23] */
  I( two_byte,      regAL,  and    ), /* AND     AL,kk              [24] */
  I( three_byte,    regAX,  and    ), /* AND     AX,jjkk            [25] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "ES"       [26] */
  I( one_byte,    regNone,  daa    ), /* DAA                        [27] */
  I( mod_reg,  modregEbGb,  sub    ), /* SUB     mem/reg,reg (byte) [28] */
  I( mod_reg,  modregEvGv,  sub    ), /* SUB     mem/reg,reg (word) [29] */
  I( mod_reg,  modregGbEb,  sub    ), /* SUB     reg,mem/reg (byte) [2A] */
  I( mod_reg,  modregGvEv,  sub    ), /* SUB     reg,mem/reg (word) [2B] */
  I( two_byte,      regAL,  sub    ), /* SUB     AL,kk              [2C] */
  I( three_byte,    regAX,  sub    ), /* SUB     AX,jjkk            [2D] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "CS"       [2E] */
  I( one_byte,    regNone,  das    ), /* DAS                        [2F] */
  I( mod_reg,  modregEbGb,  xor    ), /* XOR     mem/reg,reg (byte) [30] */
  I( mod_reg,  modregEvGv,  xor    ), /* XOR     mem/reg,reg (word) [31] */
  I( mod_reg,  modregGbEb,  xor    ), /* XOR     reg,mem/reg (byte) [32] */
  I( mod_reg,  modregGvEv,  xor    ), /* XOR     reg,mem/reg (word) [33] */
  I( two_byte,      regAL,  xor    ), /* XOR     AL,kk              [34] */
  I( three_byte,    regAX,  xor    ), /* XOR     AX,jjkk            [35] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "SS"       [36] */
  I( one_byte,    regNone,  aaa    ), /* AAA                        [37] */
  I( mod_reg,  modregEbGb,  cmp    ), /* CMP     mem/reg,reg (byte) [38] */
  I( mod_reg,  modregEvGv,  cmp    ), /* CMP     mem/reg,reg (word) [39] */
  I( mod_reg,  modregGbEb,  cmp    ), /* CMP     reg,mem/reg (byte) [3A] */
  I( mod_reg,  modregGvEv,  cmp    ), /* CMP     reg,mem/reg (word) [3B] */
  I( two_byte,      regAL,  cmp    ), /* CMP     AL,kk              [3C] */
  I( three_byte,    regAX,  cmp    ), /* CMP     AX,jjkk            [3D] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "DS"       [3E] */
  I( one_byte,    regNone,  aas    ), /* AAS                        [3F] */
  I( one_byte,      regAX,  inc    ), /* INC     AX                 [40] */
  I( one_byte,      regCX,  inc    ), /* INC     CX                 [41] */
  I( one_byte,      regDX,  inc    ), /* INC     DX                 [42] */
  I( one_byte,      regBX,  inc    ), /* INC     BX                 [43] */
  I( one_byte,      regSP,  inc    ), /* INC     SP                 [44] */
  I( one_byte,      regBP,  inc    ), /* INC     BP                 [45] */
  I( one_byte,      regSI,  inc    ), /* INC     SI                 [46] */
  I( one_byte,      regDI,  inc    ), /* INC     DI                 [47] */
  I( one_byte,      regAX,  dec    ), /* DEC     AX                 [48] */
  I( one_byte,      regCX,  dec    ), /* DEC     CX                 [49] */
  I( one_byte,      regDX,  dec    ), /* DEC     DX                 [4A] */
  I( one_byte,      regBX,  dec    ), /* DEC     BX                 [4B] */
  I( one_byte,      regSP,  dec    ), /* DEC     SP                 [4C] */
  I( one_byte,      regBP,  dec    ), /* DEC     BP                 [4D] */
  I( one_byte,      regSI,  dec    ), /* DEC     SI                 [4E] */
  I( one_byte,      regDI,  dec    ), /* DEC     DI                 [4F] */
  I( one_byte,      regAX,  push   ), /* PUSH    AX                 [50] */
  I( one_byte,      regCX,  push   ), /* PUSH    CX                 [51] */
  I( one_byte,      regDX,  push   ), /* PUSH    DX                 [52] */
  I( one_byte,      regBX,  push   ), /* PUSH    BX                 [53] */
  I( one_byte,      regSP,  push   ), /* PUSH    SP                 [54] */
  I( one_byte,      regBP,  push   ), /* PUSH    BP                 [55] */
  I( one_byte,      regSI,  push   ), /* PUSH    SI                 [56] */
  I( one_byte,      regDI,  push   ), /* PUSH    DI                 [57] */
  I( one_byte,      regAX,  pop    ), /* POP     AX                 [58] */
  I( one_byte,      regCX,  pop    ), /* POP     CX                 [59] */
  I( one_byte,      regDX,  pop    ), /* POP     DX                 [5A] */
  I( one_byte,      regBX,  pop    ), /* POP     BX                 [5B] */
  I( one_byte,      regSP,  pop    ), /* POP     SP                 [5C] */
  I( one_byte,      regBP,  pop    ), /* POP     BP                 [5D] */
  I( one_byte,      regSI,  pop    ), /* POP     SI                 [5E] */
  I( one_byte,      regDI,  pop    ), /* POP     DI                 [5F] */
  I( one_byte,    regNone,  pusha  ), /* PUSHA (PUSHAD on 386)      [60] */
  I( one_byte,    regNone,  popa   ), /* POPA (POPAD on 386)        [61] */
  I( mod_reg,  modregGvMa,  bound  ), /* BOUND                      [62] */
  I( mod_reg,  modregEwGw,  arpl   ), /* ARPL                       [63] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "FS" on 386[64] */
  I( seg_over,          0,  _      ), /* SEGMENT OVERIDE "GS" on 386[65] */
  I( opsize_over,       0,  _      ), /* OPERAND SIZE OVERIDE on 386[66] */
  I( adrsize_over,      0,  _      ), /* ADDRESS SIZE OVERIDE on 386[67] */
  I( three_byte,  regNone,  push   ), /* PUSH (immediate word)      [68] */
  I( mod_reg, modregGvEvIv, imul   ), /* IMUL reg,mem/reg,imm. word [69] */
  I( two_sbyte,         0,  push   ), /* PUSH (immed. signed byte)  [6A] */
  I( mod_reg, modregGvEvIsb,imul   ), /* IMUL reg,mem/reg,imm. byte [6B] */
  I( string_byte,      -1,  insb   ), /* INS     BYTE               [6C] */
  I( string_byte,istr_insd, insw   ), /* INS     WORD               [6D] */
  I( string_byte,      -1,  outsb  ), /* OUTS    BYTE               [6E] */
  I( string_byte,istr_outsd,outsw  ), /* OUTS    WORD               [6F] */
  I( disp8,             0,  jo     ), /* JO      <Label>            [70] */
  I( disp8,             0,  jno    ), /* JNO     <Label>            [71] */
  I( disp8,             0,  jc     ), /* JC      <Label>            [72] */
  I( disp8,             0,  jnc    ), /* JNC     <Label>            [73] */
  I( disp8,             0,  jz     ), /* JZ      <Label>            [74] */
  I( disp8,             0,  jnz    ), /* JNZ     <Label>            [75] */
  I( disp8,             0,  jbe    ), /* JBE     <Label>            [76] */
  I( disp8,             0,  ja     ), /* JA      <Label>            [77] */
  I( disp8,             0,  js     ), /* JS      <Label>            [78] */
  I( disp8,             0,  jns    ), /* JNS     <Label>            [79] */
  I( disp8,             0,  jpe    ), /* JPE     <Label>            [7A] */
  I( disp8,             0,  jpo    ), /* JPO     <Label>            [7B] */
  I( disp8,             0,  jl     ), /* JL      <Label>            [7C] */
  I( disp8,             0,  jge    ), /* JGE     <Label>            [7D] */
  I( disp8,             0,  jle    ), /* JLE     <Label>            [7E] */
  I( disp8,             0,  jg     ), /* JG      <Label>            [7F] */
  I( group1,   modregEbIub, _      ), /* Special mem/reg,kk         [80] */
  I( group1,   modregEvIv,  _      ), /* Special mem/reg,jjkk       [81] */
  I( group1,   modregEbIub, _      ), /* Special mem/reg,kk         [82] */
  I( group1,   modregEvIsb, _      ), /* Special mem/reg,kk (word)  [83] */
  I( mod_reg,  modregEbGb,  test   ), /* TEST    mem/reg,reg (byte) [84] */
  I( mod_reg,  modregEvGv,  test   ), /* TEST    mem/reg,reg (word) [85] */
  I( mod_reg,  modregEbGb,  xchg   ), /* XCHG    mem/reg,reg (byte) [86] */
  I( mod_reg,  modregEvGv,  xchg   ), /* XCHG    mem/reg,reg (word) [87] */
  I( mod_reg,  modregEbGb,  mov    ), /* MOV     mem/reg,reg (byte) [88] */
  I( mod_reg,  modregEvGv,  mov    ), /* MOV     mem/reg,reg (word) [89] */
  I( mod_reg,  modregGbEb,  mov    ), /* MOV     reg,mem/reg (byte) [8A] */
  I( mod_reg,  modregGvEv,  mov    ), /* MOV     reg,mem/reg (word) [8B] */
  I( seg_reg,  modregEwSw,  mov    ), /* MOV     mem/reg,segreg     [8C] */
  I( mod_reg,  modregGvMn,  lea    ), /* LEA     reg,mem/reg        [8D] */
  I( seg_reg,  modregSwEw,  mov    ), /* MOV     segreg,mem/reg     [8E] */
  I( mod_reg,    modregEv,  pop    ), /* POP     mem/reg (Special)  [8F] */
  I( one_byte,    regNone,  nop    ), /* NOP                        [90] */
  I( one_a,         regCX,  xchg   ), /* XCHG    AX,CX              [91] */
  I( one_a,         regDX,  xchg   ), /* XCHG    AX,DX              [92] */
  I( one_a,         regBX,  xchg   ), /* XCHG    AX,BX              [93] */
  I( one_a,         regSP,  xchg   ), /* XCHG    AX,SP              [94] */
  I( one_a,         regBP,  xchg   ), /* XCHG    AX,BP              [95] */
  I( one_a,         regSI,  xchg   ), /* XCHG    AX,SI              [96] */
  I( one_a,         regDI,  xchg   ), /* XCHG    AX,DI              [97] */
  I( conv_byte, istr_cwde,  cbw    ), /* CBW                        [98] */
  I( conv_byte,  istr_cdq,  cwd    ), /* CWD                        [99] */
  I( five_byte,         0,  call   ), /* CALL    <Label>            [9A] */
  I( wait,              0,  wait   ), /* WAIT                       [9B] */
  I( one_byte,    regNone,  pushf  ), /* PUSHF                      [9C] */
  I( one_byte,    regNone,  popf   ), /* POPF                       [9D] */
  I( one_byte,    regNone,  sahf   ), /* SAHF                       [9E] */
  I( one_byte,    regNone,  lahf   ), /* LAHF                       [9F] */
  I( three_a,           0,  mov    ), /* MOV     AL,addr            [A0] */
  I( three_a,           0,  mov    ), /* MOV     AX,addr            [A1] */
  I( three_a,           0,  mov    ), /* MOV     addr,AL            [A2] */
  I( three_a,           0,  mov    ), /* MOV     addr,AX            [A3] */
  I( string_byte,      -1,  movsb  ), /* MOVS    BYTE               [A4] */
  I( string_byte,istr_movsd,movsw  ), /* MOVS    WORD               [A5] */
  I( string_byte,      -1,  cmpsb  ), /* CMPS    BYTE               [A6] */
  I( string_byte,istr_cmpsd,cmpsw  ), /* CMPS    WORD               [A7] */
  I( two_byte,      regAL,  test   ), /* TEST    AL,kk              [A8] */
  I( three_byte,    regAX,  test   ), /* TEST    AX,jjkk            [A9] */
  I( string_byte,      -1,  stosb  ), /* STOS    BYTE               [AA] */
  I( string_byte,istr_stosd,stosw  ), /* STOS    WORD               [AB] */
  I( string_byte,      -1,  lodsb  ), /* LODS    BYTE               [AC] */
  I( string_byte,istr_lodsd,lodsw  ), /* LODS    WORD               [AD] */
  I( string_byte,      -1,  scasb  ), /* SCAS    BYTE               [AE] */
  I( string_byte,istr_scasd,scasw  ), /* SCAS    WORD               [AF] */
  I( two_byte,      regAL,  mov    ), /* MOV     AL,kk              [B0] */
  I( two_byte,      regCL,  mov    ), /* MOV     CL,kk              [B1] */
  I( two_byte,      regDL,  mov    ), /* MOV     DL,kk              [B2] */
  I( two_byte,      regBL,  mov    ), /* MOV     BL,kk              [B3] */
  I( two_byte,      regAH,  mov    ), /* MOV     AH,kk              [B4] */
  I( two_byte,      regCH,  mov    ), /* MOV     CH,kk              [B5] */
  I( two_byte,      regDH,  mov    ), /* MOV     DH,kk              [B6] */
  I( two_byte,      regBH,  mov    ), /* MOV     BH,kk              [B7] */
  I( three_byte,    regAX,  mov    ), /* MOV     AX,jjkk            [B8] */
  I( three_byte,    regCX,  mov    ), /* MOV     CX,jjkk            [B9] */
  I( three_byte,    regDX,  mov    ), /* MOV     DX,jjkk            [BA] */
  I( three_byte,    regBX,  mov    ), /* MOV     BX,jjkk            [BB] */
  I( three_byte,    regSP,  mov    ), /* MOV     SP,jjkk            [BC] */
  I( three_byte,    regBP,  mov    ), /* MOV     BP,jjkk            [BD] */
  I( three_byte,    regSI,  mov    ), /* MOV     SI,jjkk            [BE] */
  I( three_byte,    regDI,  mov    ), /* MOV     DI,jjkk            [BF] */
  I( group2,   modregEbIub, _      ), /* Special mem/reg,kk  (byte) [C0] */
  I( group2,   modregEvIub, _      ), /* Special mem/reg,kk  (word) [C1] */
  I( three_byte,  regNone,  ret    ), /* RET     jjkk               [C2] */
  I( one_byte,    regNone,  ret    ), /* RET                        [C3] */
  I( mod_reg,  modregGvMp,  les    ), /* LES     reg,mem            [C4] */
  I( mod_reg,  modregGvMp,  lds    ), /* LDS     reg,mem            [C5] */
// dato che non c'e' estensione considera unsigned
  I( mov_special, modregEbIub, mov ), /* MOV     mem,kk   (Special) [C6] */
  I( mov_special, modregEvIv,  mov ), /* MOV     mem,jjkk (Special) [C7] */
  I( enter,             0,  enter  ), /* ENTER   im. word,im. byte  [C8] */
  I( one_byte,    regNone,  leave  ), /* LEAVE                      [C9] */
  I( three_byte,  regNone,  retf   ), /* RETF    jjkk               [CA] */
  I( one_byte,    regNone,  retf   ), /* RETF                       [CB] */
  I( one_byte,    regNone,  int3   ), /* INT     3                  [CC] */
  I( two_byte,    regNone,  int    ), /* INT     type               [CD] */
  I( one_byte,    regNone,  into   ), /* INTO                       [CE] */
  I( one_byte,    regNone,  iret   ), /* IRET                       [CF] */
  I( group2,    modregEb1,  _      ), /* Special mem/reg,1  (byte)  [D0] */
  I( group2,    modregEv1,  _      ), /* Special mem/reg,1  (word)  [D1] */
  I( group2,   modregEbCL,  _      ), /* Special mem/reg,CL (byte)  [D2] */
  I( group2,   modregEvCL,  _      ), /* Special mem/reg,CL (word)  [D3] */
  I( two_bcd,           0,  aam    ), /* AAM                        [D4] */
  I( two_bcd,           0,  aad    ), /* AAD                        [D5] */
  I( stub,              0,  _      ), /* not used                   [D6] */
  I( one_byte,    regNone,  xlat   ), /* XLAT                       [D7] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [D8] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [D9] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DA] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DB] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DC] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DD] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DE] */
  I( esc,               0,  esc    ), /* ESC  (Special)             [DF] */
  I( disp8,             0,  loopnz ), /* LOOPNZ                     [E0] */
  I( disp8,             0,  loopz  ), /* LOOPZ                      [E1] */
  I( disp8,             0,  loop   ), /* LOOP                       [E2] */
  I( disp8,             0,  jcxz   ), /* JCXZ                       [E3] */
  I( two_byte,      regAL,  in     ), /* IN      AL,kk              [E4] */
  I( two_byte,      regAX,  in     ), /* IN      AX,kk              [E5] */
  I( two_a,             0,  out    ), /* OUT     kk,AL              [E6] */
  I( two_a,             0,  out    ), /* OUT     kk,AX              [E7] */
  I( disp16,            0,  call   ), /* CALL    <Label>            [E8] */
  I( disp16,            0,  jmp    ), /* JMP     <Label>            [E9] */
  I( five_byte,         0,  jmp    ), /* JMP     <Label>            [EA] */
  I( disp8,             0,  jmp    ), /* JMP     SHORT <Label>      [EB] */
  I( in_out,        regDX,  in     ), /* IN      AL,DX              [EC] */
  I( in_out,        regDX,  in     ), /* IN      AX,DX              [ED] */
  I( in_out,        regDX,  out    ), /* OUT     DX,AL              [EE] */
  I( in_out,        regDX,  out    ), /* OUT     DX,AX              [EF] */
  I( prefix,            0,  lock   ), /* LOCK                       [F0] */
  I( stub,              0,  _      ), /* not used                   [F1] */
  I( prefixf2,          0,  repne  ), /* REPNZ                      [F2] */
  I( prefixf3,          0,  rep    ), /* REPZ                       [F3] */
  I( one_byte,    regNone,  hlt    ), /* HLT                        [F4] */
  I( one_byte,    regNone,  cmc    ), /* CMC                        [F5] */
  I( group3,     modregEb,  _      ), /* Special (byte) (Group 3)   [F6] */
  I( group3,     modregEv,  _      ), /* Special (word) (Group 3)   [F7] */
  I( one_byte,    regNone,  clc    ), /* CLC                        [F8] */
  I( one_byte,    regNone,  stc    ), /* STC                        [F9] */
  I( one_byte,    regNone,  cli    ), /* CLI                        [FA] */
  I( one_byte,    regNone,  sti    ), /* STI                        [FB] */
  I( one_byte,    regNone,  cld    ), /* CLD                        [FC] */
  I( one_byte,    regNone,  std    ), /* STD                        [FD] */
  I( group4,     modregEb,  _      ), /* Special (Group 4)          [FE] */
  I( group5,     modregEv,  _      )  /* Special (Group 5)          [FF] */
};

const InstructionDecoder::InstTable InstructionDecoder::exInstrTable[] = {
  I( group6,       modregEw,  _        ), /* Spec. (word) (grp 6)   [0F] [00] */
  I( group7,       modregMs,  _        ), /* Spec. (dword) (grp 7)  [0F] [01] */
  I( mod_reg,    modregGvEw,  lar      ), /* LAR                    [0F] [02] */
  I( mod_reg,    modregGvEw,  lsl      ), /* LSL                    [0F] [03] */
  I( stub,                0,  _        ), /* not used               [0F] [04] */
  I( stub,                0,  _        ), /* not used               [0F] [05] */
  I( one_byte,      regNone,  clts     ), /* CLTS                   [0F] [06] */
  I( stub,                0,  _        ), /* not used               [0F] [07] */
  I( one_byte,      regNone,  invd     ), /* INVD (486+)            [0F] [08] */
  I( one_byte,      regNone,  wbinvd   ), /* WBINVD (486+)          [0F] [09] */
  I( stub,                0,  _        ), /* not used               [0F] [0A] */
  I( stub,                0,  ud2      ), /* ud2                    [0F] [0B] */
  I( stub,                0,  _        ), /* not used               [0F] [0C] */
  I( prefetch_k63d,       0,  prefetch ), /* PREFETCH               [0F] [0D] */
  I( one_byte_k63d, regNone,  femms    ), /* FEMMS                  [0F] [0E] */
  I( k6_3dnow,   modregPqQq,  _        ), /* 3DNow!                 [0F] [0F] */
  I( mod_xmm,  modregVpsWps,  movups   ), /* movups   reg,mem/reg   [0F] [10] */
  I( mod_xmm,  modregWpsVps,  movups   ), /* movups   mem/reg,reg   [0F] [11] */
  I( mod_xmm,    modregVqWq,  movlps   ), /* movlps   reg,mem       [0F] [12] */
  I( mod_xmm,    modregWqVq,  movlps   ), /* movlps   mem,reg       [0F] [13] */
  // changed from VpsWq to VpsWps as address must be aligned to 16byte bound
  I( mod_xmm,  modregVpsWps,  unpcklps ), /* unpcklps reg,mem/reg   [0F] [14] */
  I( mod_xmm,  modregVpsWps,  unpckhps ), /* unpckhps reg,mem/reg   [0F] [15] */
  I( mod_xmm,    modregVqWq,  movhps   ), /* movhps   reg,mem       [0F] [16] */
  I( mod_xmm,    modregWqVq,  movhps   ), /* movhps   mem,reg       [0F] [17] */
  I( group16,             0,  _        ), /* Special (Group 16)     [0F] [18] */
  I( stub,                0,  _        ), /* not used               [0F] [19] */
  I( stub,                0,  _        ), /* not used               [0F] [1A] */
  I( stub,                0,  _        ), /* not used               [0F] [1B] */
  I( stub,                0,  _        ), /* not used               [0F] [1C] */
  I( stub,                0,  _        ), /* not used               [0F] [1D] */
  I( stub,                0,  _        ), /* not used               [0F] [1E] */
  I( stub,                0,  _        ), /* not used               [0F] [1F] */
  I( mov_creg, movcregR32CRx, mov      ), /* MOV      r32,CRx       [0F] [20] */
  I( mov_creg, movcregR32DRx,  mov     ), /* MOV      r32,DRx       [0F] [21] */
  I( mov_creg, movcregCRxr32,  mov     ), /* MOV      CRx,r32       [0F] [22] */
  I( mov_creg, movcregDRxr32,  mov     ), /* MOV      DRx,r32       [0F] [23] */
  I( mov_creg, movcregR32TRx,  mov     ), /* MOV r32,TRx 486only!!  [0F] [24] */
  I( stub,                0,     _     ), /* not used               [0F] [25] */
  I( mov_creg, movcregTRxr32,  mov     ), /* MOV TRx,r32 486only!!  [0F] [26] */
  I( stub,                0,     _     ), /* not used               [0F] [27] */
  I( mod_xmm,  modregVpsWps,   movaps  ), /* movaps   reg,mem/reg   [0F] [28] */
  I( mod_xmm,  modregWpsVps,   movaps  ), /* movaps   mem/reg,reg   [0F] [29] */
  // this is VpsQq, not WpsQq
  I( mod_xmm,  modregVpsQq,    cvtpi2ps), /* cvtpi2ps reg,mem/reg   [0F] [2A] */
  I( mod_xmm,  modregWpsVps,   movntps ), /* movntps  mem/reg,reg   [0F] [2B] */
  I( mod_xmm,    modregPqWq,  cvttps2pi), /* cvttps2pi reg,mem/reg  [0F] [2C] */
  // cvtps2pi mmx,xmm/mem64, this is QqWq, not QqWps
  I( mod_xmm,    modregPqWq,   cvtps2pi), /* cvtps2pi reg,mem/reg   [0F] [2D] */
  I( mod_xmm,  modregVssWss,   ucomiss ), /* ucomiss  reg,mem/reg   [0F] [2E] */
  // this is VssWss, not VpsWps
  I( mod_xmm,  modregVssWss,   comiss  ), /* comiss   reg,mem/reg   [0F] [2F] */
  I( one_byte,      regNone,  wrmsr    ), /* WRMSR                  [0F] [30] */
  I( one_byte,      regNone,  rdtsc    ), /* RDTSC                  [0F] [31] */
  I( one_byte,      regNone,  rdmsr    ), /* RDMSR                  [0F] [32] */
  I( one_byte,      regNone,  rdpmc    ), /* RDPMC                  [0F] [33] */
  I( one_byte,      regNone,  sysenter ), /* SYSENTER               [0F] [34] */
  I( one_byte,      regNone,  sysexit  ), /* SYSEXIT                [0F] [35] */
  I( stub,                0,     _     ), /* not used               [0F] [36] */
  I( one_byte,      regNone,  getsec   ), /* GETSEC                 [0F] [37] */
  I( stub,                0,     _     ), /* (3-byte escape)        [0F] [38] */
  I( stub,                0,     _     ), /* not used               [0F] [39] */
  I( stub,                0,     _     ), /* (3-byte escape)        [0F] [3A] */
  I( stub,                0,     _     ), /* not used               [0F] [3B] */
  I( stub,                0,     _     ), /* not used               [0F] [3C] */
  I( stub,                0,     _     ), /* not used               [0F] [3D] */
  I( stub,                0,     _     ), /* not used               [0F] [3E] */
  I( stub,                0,     _     ), /* not used               [0F] [3F] */
  I( mod_reg,    modregGvEv,  cmovo    ), /* CMOVO   reg,mem/reg    [0F] [40] */
  I( mod_reg,    modregGvEv,  cmovno   ), /* CMOVNO  reg,mem/reg    [0F] [41] */
  I( mod_reg,    modregGvEv,  cmovc    ), /* CMOVC   reg,mem/reg    [0F] [42] */
  I( mod_reg,    modregGvEv,  cmovnc   ), /* CMOVNC  reg,mem/reg    [0F] [43] */
  I( mod_reg,    modregGvEv,  cmovz    ), /* CMOVZ   reg,mem/reg    [0F] [44] */
  I( mod_reg,    modregGvEv,  cmovnz   ), /* CMOVNZ  reg,mem/reg    [0F] [45] */
  I( mod_reg,    modregGvEv,  cmovbe   ), /* CMOVBE  reg,mem/reg    [0F] [46] */
  I( mod_reg,    modregGvEv,  cmova    ), /* CMOVA   reg,mem/reg    [0F] [47] */
  I( mod_reg,    modregGvEv,  cmovs    ), /* CMOVS   reg,mem/reg    [0F] [48] */
  I( mod_reg,    modregGvEv,  cmovns   ), /* CMOVNS  reg,mem/reg    [0F] [49] */
  I( mod_reg,    modregGvEv,  cmovpe   ), /* CMOVPE  reg,mem/reg    [0F] [4A] */
  I( mod_reg,    modregGvEv,  cmovpo   ), /* CMOVPO  reg,mem/reg    [0F] [4B] */
  I( mod_reg,    modregGvEv,  cmovl    ), /* CMOVL   reg,mem/reg    [0F] [4C] */
  I( mod_reg,    modregGvEv,  cmovge   ), /* CMOVGE  reg,mem/reg    [0F] [4D] */
  I( mod_reg,    modregGvEv,  cmovle   ), /* CMOVLE  reg,mem/reg    [0F] [4E] */
  I( mod_reg,    modregGvEv,  cmovg    ), /* CMOVG   reg,mem/reg    [0F] [4F] */
  // !!! can be memory ?
  I( mod_xmm,   modregEdVps,  movmskps ), /* movmskps mem/reg,reg   [0F] [50] */
  I( mod_xmm,  modregVpsWps,  sqrtps   ), /* sqrtps   reg,mem/reg   [0F] [51] */
  I( mod_xmm,  modregVpsWps,  rsqrtps  ), /* rsqrtps  reg,mem/reg   [0F] [52] */
  I( mod_xmm,  modregVpsWps,  rcpps    ), /* rcpps    reg,mem/reg   [0F] [53] */
  I( mod_xmm,  modregVpsWps,  andps    ), /* andps    reg,mem/reg   [0F] [54] */
  I( mod_xmm,  modregVpsWps,  andnps   ), /* andnps   reg,mem/reg   [0F] [55] */
  I( mod_xmm,  modregVpsWps,  orps     ), /* orps     reg,mem/reg   [0F] [56] */
  I( mod_xmm,  modregVpsWps,  xorps    ), /* xorps    reg,mem/reg   [0F] [57] */
  I( mod_xmm,  modregVpsWps,  addps    ), /* addps    reg,mem/reg   [0F] [58] */
  I( mod_xmm,  modregVpsWps,  mulps    ), /* mulps    reg,mem/reg   [0F] [59] */
  I( mod_sse2, modregVpdWps,  cvtps2pd ), /* cvtps2pd               [0F] [5A] */
  I( mod_sse2, modregVpsWdq,  cvtdq2ps ), /* cvtdq2ps               [0F] [5B] */
  I( mod_xmm,  modregVpsWps,  subps    ), /* subps    reg,mem/reg   [0F] [5C] */
  I( mod_xmm,  modregVpsWps,  minps    ), /* minps    reg,mem/reg   [0F] [5D] */
  I( mod_xmm,  modregVpsWps,  divps    ), /* divps    reg,mem/reg   [0F] [5E] */
  I( mod_xmm,  modregVpsWps,  maxps    ), /* maxps    reg,mem/reg   [0F] [5F] */
  I( mod_reg,    modregPqQd, punpcklbw ), /* punpcklbw              [0F] [60] */
  I( mod_reg,    modregPqQd, punpcklwd ), /* punpcklwd              [0F] [61] */
  I( mod_reg,    modregPqQd, punpckldq ), /* punpckldq              [0F] [62] */
  I( mod_reg,    modregPqQq, packsswb  ), /* packsswb               [0F] [63] */
  I( mod_reg,    modregPqQq, pcmpgtb   ), /* pcmpgtb                [0F] [64] */
  I( mod_reg,    modregPqQq, pcmpgtw   ), /* pcmpgtw                [0F] [65] */
  I( mod_reg,    modregPqQq, pcmpgtd   ), /* pcmpgtd                [0F] [66] */
  I( mod_reg,    modregPqQq, packuswb  ), /* packuswb               [0F] [67] */
  I( mod_reg,    modregPqQq, punpckhbw ), /* punpckhbw              [0F] [68] */
  I( mod_reg,    modregPqQq, punpckhwd ), /* punpckhwd              [0F] [69] */
  I( mod_reg,    modregPqQq, punpckhdq ), /* punpckhdq              [0F] [6A] */
  I( mod_reg,    modregPqQq, packssdw  ), /* packssdw               [0F] [6B] */
  I( stub,                0,     _     ), /* not used               [0F] [6C] */
  I( stub,                0,     _     ), /* not used               [0F] [6D] */
  I( mod_reg,    modregPdEd, movd      ), /* movd                   [0F] [6E] */
  I( mod_reg,    modregPqQq, movq      ), /* movq                   [0F] [6F] */
  I( mod_reg_p3, modregPqQqIub,pshufw  ), /* pshufw                 [0F] [70] */
  I( groupA,     modregPqIub,    _     ), /* Special (Group A)      [0F] [71] */
  I( groupA,     modregPqIub,    _     ), /* Special (Group A)      [0F] [72] */
  I( groupA,     modregPqIub,    _     ), /* Special (Group A)      [0F] [73] */
  I( mod_reg,    modregPqQq,  pcmpeqb  ), /* pcmpeqb                [0F] [74] */
  I( mod_reg,    modregPqQq,  pcmpeqw  ), /* pcmpeqw                [0F] [75] */
  I( mod_reg,    modregPqQq,  pcmpeqd  ), /* pcmpeqd                [0F] [76] */
  I( one_byte,      regNone,  emms     ), /* EMMS                   [0F] [77] */
  // MMX UD (tutti e 6 gli opcode)
  I( stub,                0,     _     ), /* (VMREAD)               [0F] [78] */
  I( stub,                0,     _     ), /* (VMWRITE)              [0F] [79] */
  I( stub,                0,     _     ), /* not used               [0F] [7A] */
  I( stub,                0,     _     ), /* not used               [0F] [7B] */
  I( stub,                0,     _     ), /* haddps/d               [0F] [7C] */
  I( stub,                0,     _     ), /* hsubps/d               [0F] [7D] */
  I( mod_reg,    modregEdPd,     movd  ), /* movd                   [0F] [7E] */
  I( mod_reg,    modregQqPq,     movq  ), /* movq                   [0F] [7F] */
  I( disp16,              0,  jo       ), /* JO       <Label>       [0F] [80] */
  I( disp16,              0,  jno      ), /* JNO      <Label>       [0F] [81] */
  I( disp16,              0,  jc       ), /* JC       <Label>       [0F] [82] */
  I( disp16,              0,  jnc      ), /* JNC      <Label>       [0F] [83] */
  I( disp16,              0,  jz       ), /* JZ       <Label>       [0F] [84] */
  I( disp16,              0,  jnz      ), /* JNZ      <Label>       [0F] [85] */
  I( disp16,              0,  jbe      ), /* JBE      <Label>       [0F] [86] */
  I( disp16,              0,  ja       ), /* JA       <Label>       [0F] [87] */
  I( disp16,              0,  js       ), /* JS       <Label>       [0F] [88] */
  I( disp16,              0,  jns      ), /* JNS      <Label>       [0F] [89] */
  I( disp16,              0,  jpe      ), /* JPE      <Label>       [0F] [8A] */
  I( disp16,              0,  jpo      ), /* JPO      <Label>       [0F] [8B] */
  I( disp16,              0,  jl       ), /* JL       <Label>       [0F] [8C] */
  I( disp16,              0,  jge      ), /* JGE      <Label>       [0F] [8D] */
  I( disp16,              0,  jle      ), /* JLE      <Label>       [0F] [8E] */
  I( disp16,              0,  jg       ), /* JG       <Label>       [0F] [8F] */
  I( mod_reg,      modregEb,  seto     ), /* SETO     mem/reg       [0F] [90] */
  I( mod_reg,      modregEb,  setno    ), /* SETNO    mem/reg       [0F] [91] */
  I( mod_reg,      modregEb,  setc     ), /* SETC     mem/reg       [0F] [92] */
  I( mod_reg,      modregEb,  setnc    ), /* SETNC    mem/reg       [0F] [93] */
  I( mod_reg,      modregEb,  setz     ), /* SETZ     mem/reg       [0F] [94] */
  I( mod_reg,      modregEb,  setnz    ), /* SETNZ    mem/reg       [0F] [95] */
  I( mod_reg,      modregEb,  setbe    ), /* SETBE    mem/reg       [0F] [96] */
  I( mod_reg,      modregEb,  seta     ), /* SETA     mem/reg       [0F] [97] */
  I( mod_reg,      modregEb,  sets     ), /* SETS     mem/reg       [0F] [98] */
  I( mod_reg,      modregEb,  setns    ), /* SETNS    mem/reg       [0F] [99] */
  I( mod_reg,      modregEb,  setpe    ), /* SETPE    mem/reg       [0F] [9A] */
  I( mod_reg,      modregEb,  setpo    ), /* SETPO    mem/reg       [0F] [9B] */
  I( mod_reg,      modregEb,  setl     ), /* SETL     mem/reg       [0F] [9C] */
  I( mod_reg,      modregEb,  setge    ), /* SETGE    mem/reg       [0F] [9D] */
  I( mod_reg,      modregEb,  setle    ), /* SETLE    mem/reg       [0F] [9E] */
  I( mod_reg,      modregEb,  setg     ), /* SETG     mem/reg       [0F] [9F] */
  I( one_byte,        regFS,  push     ), /* PUSH FS                [0F] [A0] */
  I( one_byte,        regFS,  pop      ), /* POP FS                 [0F] [A1] */
  I( one_byte,      regNone,  cpuid    ), /* CPUID                  [0F] [A2] */
  I( mod_reg,    modregEvGv,  bt       ), /* BT       mem/reg,reg   [0F] [A3] */
  I( mod_reg,  modregEvGvIub, shld     ), /* SHLD     mem,reg,immb  [0F] [A4] */
  I( mod_reg,  modregEvGvCL,  shld     ), /* SHLD     mem,reg,cl    [0F] [A5] */
  I( stub,                0,     _     ), /* not used               [0F] [A6] */
  I( stub,                0,     _     ), /* not used               [0F] [A7] */
  I( one_byte,        regGS,  push     ), /* PUSH GS                [0F] [A8] */
  I( one_byte,        regGS,  pop      ), /* POP GS                 [0F] [A9] */
  I( one_byte,      regNone,  rsm      ), /* RSM                    [0F] [AA] */
  I( mod_reg,    modregEvGv,  bts      ), /* BTS      mem/reg,reg   [0F] [AB] */
  I( mod_reg,  modregEvGvIub, shrd     ), /* SHRD     mem,reg,immb  [0F] [AC] */
  I( mod_reg,  modregEvGvCL,  shrd     ), /* SHRD     mem,reg,cl    [0F] [AD] */
  I( group15,             0,     _     ), /* Special (Group 15)     [0F] [AE] */
  I( mod_reg,    modregGvEv,  imul     ), /* IMUL     reg,mem/reg   [0F] [AF] */
  I( mod_reg,    modregEbGb,  cmpxchg  ), /* CMPXCHG                [0F] [B0] */
  I( mod_reg,    modregEvGv,  cmpxchg  ), /* CMPXCHG                [0F] [B1] */
  I( mod_reg,    modregGvMp,  lss      ), /* LSS      reg,mem       [0F] [B2] */
  I( mod_reg,    modregEvGv,  btr      ), /* BTR      mem/reg,reg   [0F] [B3] */
  I( mod_reg,    modregGvMp,  lfs      ), /* LFS      reg,mem       [0F] [B4] */
  I( mod_reg,    modregGvMp,  lgs      ), /* LGS      reg,mem       [0F] [B5] */
  I( mod_reg,    modregGvEb,  movzx    ), /* MOVZX    reg,mem (b)   [0F] [B6] */
  I( mod_reg,    modregGvEw,  movzx    ), /* MOVZX    reg,mem (w)   [0F] [B7] */
  I( disp16,              0,  jmpe     ), /* JMPE (IA-64)           [0F] [B8] */
// gruppo 10, gruppo invalido
  I( stub,                0,  ud       ), /* ud                     [0F] [B9] */
// !!! unsigned ??
  I( group8,    modregEvIub,    _      ), /* Spec. (byte) (Group 8) [0F] [BA] */
  I( mod_reg,    modregEvGv,  btc      ), /* BTC      mem/reg,reg   [0F] [BB] */
  I( mod_reg,    modregGvEv,  bsf      ), /* BSF      reg,mem/reg   [0F] [BC] */
  I( mod_reg,    modregGvEv,  bsr      ), /* BSR      reg,mem/reg   [0F] [BD] */
  I( mod_reg,    modregGvEb,  movsx    ), /* MOVSX    reg,mem (b)   [0F] [BE] */
  I( mod_reg,    modregGvEw,  movsx    ), /* MOVSX    reg,mem (w)   [0F] [BF] */
  I( mod_reg,    modregEbGb,  xadd     ), /* XADD (486 only)        [0F] [C0] */
  I( mod_reg,    modregEvGv,  xadd     ), /* XADD (486 only)        [0F] [C1] */
  // !!! Ib should be checked, change instruction from Ib codes !!!
  I( mod_xmm, modregVpsWpsIub,cmpps    ), /* cmpps reg,mem/reg,immb [0F] [C2] */
  // !!! SSE2
  I( mod_reg_p3, modregMdGd,  movnti   ), /* movnti   mem32,reg32   [0F] [C3] */
// registro 32 o memoria a 16, cambiato nel codice
  I( mod_reg_p3, modregPqEdIub,pinsrw  ), /* pinsrw                 [0F] [C4] */
// in r_m registro mmx, in reg registro intero, eccezione se memoria
// dovrebbe essere GdPqIub forzato in codice
  I( mod_reg_p3, modregGdQqIub,pextrw  ), /* pextrw                 [0F] [C5] */
  I( mod_xmm, modregVpsWpsIub,shufps   ), /* shufps reg,mem/reg,immb[0F] [C6] */
  I( group9,       modregMq,     _     ), /* Special (Group 9)      [0F] [C7] */
  I( one_byte,        regAX,  bswap    ), /* BSWAP AX (486 only)    [0F] [C8] */
  I( one_byte,        regCX,  bswap    ), /* BSWAP CX (486 only)    [0F] [C9] */
  I( one_byte,        regDX,  bswap    ), /* BSWAP DX (486 only)    [0F] [CA] */
  I( one_byte,        regBX,  bswap    ), /* BSWAP BX (486 only)    [0F] [CB] */
  I( one_byte,        regSP,  bswap    ), /* BSWAP SP (486 only)    [0F] [CC] */
  I( one_byte,        regBP,  bswap    ), /* BSWAP BP (486 only)    [0F] [CD] */
  I( one_byte,        regSI,  bswap    ), /* BSWAP SI (486 only)    [0F] [CE] */
  I( one_byte,        regDI,  bswap    ), /* BSWAP DI (486 only)    [0F] [CF] */
  I( stub,                0,     _     ), /* addsubps/d             [0F] [D0] */
  I( mod_reg,    modregPqQq,     psrlw ), /* psrlw                  [0F] [D1] */
  I( mod_reg,    modregPqQq,     psrld ), /* psrld                  [0F] [D2] */
  I( mod_reg,    modregPqQq,     psrlq ), /* psrlq                  [0F] [D3] */
  // !!! SSE2
  I( mod_reg_p3, modregPqQq,     paddq ), /* paddq                  [0F] [D4] */
  I( mod_reg,    modregPqQq,    pmullw ), /* pmullw                 [0F] [D5] */
  I( stub,                0,     _     ), /* movq/movq2dq/movdq2q   [0F] [D6] */
// in r_m registro mmx, in reg registro intero, eccezione se memoria
// dovrebbe essere GdPq , fissato in codice
  I( mod_reg_p3, modregGdQq,  pmovmskb ), /* pmovmskb               [0F] [D7] */
  I( mod_reg,    modregPqQq,   psubusb ), /* psubusb                [0F] [D8] */
  I( mod_reg,    modregPqQq,   psubusw ), /* psubusw                [0F] [D9] */
  I( mod_reg_p3, modregPqQq,   pminub  ), /* pminub                 [0F] [DA] */
  I( mod_reg,    modregPqQq,   pand    ), /* pand                   [0F] [DB] */
  I( mod_reg,    modregPqQq,   paddusb ), /* paddusb                [0F] [DC] */
  I( mod_reg,    modregPqQq,   paddusw ), /* paddusw                [0F] [DD] */
  I( mod_reg_p3, modregPqQq,   pmaxub  ), /* pmaxub                 [0F] [DE] */
  I( mod_reg,    modregPqQq,   pandn   ), /* pandn                  [0F] [DF] */
  I( mod_reg_p3, modregPqQq,   pavgb   ), /* pavgb                  [0F] [E0] */
  I( mod_reg,    modregPqQq,   psraw   ), /* psraw                  [0F] [E1] */
  I( mod_reg,    modregPqQq,   psrad   ), /* psrad                  [0F] [E2] */
  I( mod_reg_p3, modregPqQq,   pavgw   ), /* pavgw                  [0F] [E3] */
  I( mod_reg_p3, modregPqQq,   pmulhuw ), /* pmulhuw                [0F] [E4] */
  I( mod_reg,    modregPqQq,   pmulhw  ), /* pmulhw                 [0F] [E5] */
  I( stub,                0,     _     ), /* cvt*                   [0F] [E6] */
  I( mod_reg_p3, modregQqPq,   movntq  ), /* movntq                 [0F] [E7] */
  I( mod_reg,    modregPqQq,    psubsb ), /* psubsb                 [0F] [E8] */
  I( mod_reg,    modregPqQq,    psubsw ), /* psubsw                 [0F] [E9] */
  I( mod_reg_p3, modregPqQq,    pminsw ), /* pminsw                 [0F] [EA] */
  I( mod_reg,    modregPqQq,    por    ), /* por                    [0F] [EB] */
  I( mod_reg,    modregPqQq,    paddsb ), /* paddsb                 [0F] [EC] */
  I( mod_reg,    modregPqQq,    paddsw ), /* paddsw                 [0F] [ED] */
  I( mod_reg_p3, modregPqQq,    pmaxsw ), /* pmaxsw                 [0F] [EE] */
  I( mod_reg,    modregPqQq,    pxor   ), /* pxor                   [0F] [EF] */
  I( stub,                0,     _     ), /* lddqu                  [0F] [F0] */
  I( mod_reg,    modregPqQq,   psllw   ), /* psllw                  [0F] [F1] */
  I( mod_reg,    modregPqQq,   pslld   ), /* pslld                  [0F] [F2] */
  I( mod_reg,    modregPqQq,   psllq   ), /* psllq                  [0F] [F3] */
  I( mod_reg_p3, modregPqQq,   pmuludq ), /* pmuludq                [0F] [F4] */
  I( mod_reg,    modregPqQq,   pmaddwd ), /* pmaddwd                [0F] [F5] */
  I( mod_reg_p3, modregPqQq,   psadbw  ), /* psadbw                 [0F] [F6] */
// reg = destination, r/m = source, eccezione se memoria
// dovrebbe essere PpiQpi, test in codice
  I( mod_reg_p3, modregPqQq,  maskmovq ), /* maskmovq               [0F] [F7] */
  I( mod_reg,    modregPqQq,     psubb ), /* psubb                  [0F] [F8] */
  I( mod_reg,    modregPqQq,     psubw ), /* psubw                  [0F] [F9] */
  I( mod_reg,    modregPqQq,     psubd ), /* psubd                  [0F] [FA] */
  I( mod_reg_p3, modregPqQq,     psubq ), /* psubq                  [0F] [FB] */
  I( mod_reg,    modregPqQq,     paddb ), /* paddb                  [0F] [FC] */
  I( mod_reg,    modregPqQq,     paddw ), /* paddw                  [0F] [FD] */
  I( mod_reg,    modregPqQq,     paddd ), /* paddd                  [0F] [FE] */
  I( stub,                0,     _     )  /* not used               [0F] [FF] */
#undef I
};

inline void InstructionDecoder::CheckTables()
{
  // produce a warning on some compiler, ignore it
  COMPILE_CHECK(instrLenCheck,
    (sizeof(InstructionDecoder::instrTable) == (sizeof(InstructionDecoder::instrTable[0])*256))
    && (sizeof(InstructionDecoder::exInstrTable) == (sizeof(InstructionDecoder::exInstrTable[0])*256))
  );
}

COMPILE_CHECK(intSizeCheck,
  (sizeof(int) >= 4) && (num_instructions <= (1<<23))
);

const int esc_inst[] = {
  DEFINE_FPI(istr_fadd,fpiTypeSingle),          /* FADD   (DWord Ptr) [00] */
  DEFINE_FPI(istr_fmul,fpiTypeSingle),          /* FMUL   (DWord Ptr) [01] */
  DEFINE_FPI(istr_fcom,fpiTypeSingle),          /* FCOM   (DWord Ptr) [02] */
  DEFINE_FPI(istr_fcomp,fpiTypeSingle),         /* FCOMP  (DWord Ptr) [03] */
  DEFINE_FPI(istr_fsub,fpiTypeSingle),          /* FSUB   (DWord Ptr) [04] */
  DEFINE_FPI(istr_fsubr,fpiTypeSingle),         /* FSUBR  (DWord Ptr) [05] */
  DEFINE_FPI(istr_fdiv,fpiTypeSingle),          /* FDIV   (DWord Ptr) [06] */
  DEFINE_FPI(istr_fdivr,fpiTypeSingle),         /* FDIVR  (DWord Ptr) [07] */

  DEFINE_FPI(istr_fld,fpiTypeSingle),           /* FLD    (DWord Ptr) [08] */
  DEFINE_FPI(-1,0),                             /* ESC 09h            [09] */
  DEFINE_FPI(istr_fst,fpiTypeSingle),           /* FST    (DWord Ptr) [0A] */
  DEFINE_FPI(istr_fstp,fpiTypeSingle),          /* FSTP   (DWord Ptr) [0B] */
  DEFINE_FPI(istr_fldenv,fpiTypeEnv),           /* FLDENV             [0C] */
  DEFINE_FPI(istr_fldcw,fpiTypeWord),           /* FLDCW              [0D] */
  DEFINE_FPI(istr_fnstenv,fpiTypeEnv),          /* FSTENV             [0E] */
  DEFINE_FPI(istr_fnstcw,fpiTypeWord),          /* FSTCW              [0F] */

  DEFINE_FPI(istr_fiadd,fpiTypeSInt),           /* FIADD  (DWord Ptr) [10] */
  DEFINE_FPI(istr_fimul,fpiTypeSInt),           /* FIMUL  (DWord Ptr) [11] */
  DEFINE_FPI(istr_ficom,fpiTypeSInt),           /* FICOM  (DWord Ptr) [12] */
  DEFINE_FPI(istr_ficomp,fpiTypeSInt),          /* FICOMP (DWord Ptr) [13] */
  DEFINE_FPI(istr_fisub,fpiTypeSInt),           /* FISUB  (DWord Ptr) [14] */
  DEFINE_FPI(istr_fisubr,fpiTypeSInt),          /* FISUBR (DWord Ptr) [15] */
  DEFINE_FPI(istr_fidiv,fpiTypeSInt),           /* FIDIV  (DWord Ptr) [16] */
  DEFINE_FPI(istr_fidivr,fpiTypeSInt),          /* FIDIVR (DWord Ptr) [17] */

  DEFINE_FPI(istr_fild,fpiTypeSInt),            /* FILD   (DWord Ptr) [18] */
  DEFINE_FPI(istr_fisttp,fpiTypeSInt),          /* FISTTP             [19] */
  DEFINE_FPI(istr_fist,fpiTypeSInt),            /* FIST   (DWord Ptr) [1A] */
  DEFINE_FPI(istr_fistp,fpiTypeSInt),           /* FISTP  (DWord Ptr) [1B] */
  DEFINE_FPI(-1,0),                             /* ESC 1Ch            [1C] */
  DEFINE_FPI(istr_fld,fpiTypeExtended),         /* FLD    (TByte Ptr) [1D] */
  DEFINE_FPI(-1,0),                             /* ESC 1Eh            [1E] */
  DEFINE_FPI(istr_fstp,fpiTypeExtended),        /* FSTP   (TByte Ptr) [1F] */

  DEFINE_FPI(istr_fadd,fpiTypeDouble),          /* FADD   (QWord Ptr) [20] */
  DEFINE_FPI(istr_fmul,fpiTypeDouble),          /* FMUL   (QWord Ptr) [21] */
  DEFINE_FPI(istr_fcom,fpiTypeDouble),          /* FCOM   (QWord Ptr) [22] */
  DEFINE_FPI(istr_fcomp,fpiTypeDouble),         /* FCOMP  (QWord Ptr) [23] */
  DEFINE_FPI(istr_fsub,fpiTypeDouble),          /* FSUB   (QWord Ptr) [24] */
  DEFINE_FPI(istr_fsubr,fpiTypeDouble),         /* FSUBR  (QWord Ptr) [25] */
  DEFINE_FPI(istr_fdiv,fpiTypeDouble),          /* FDIV   (QWord Ptr) [26] */
  DEFINE_FPI(istr_fdivr,fpiTypeDouble),         /* FDIVR  (QWord Ptr) [27] */

  DEFINE_FPI(istr_fld,fpiTypeDouble),           /* FLD    (QWord Ptr) [28] */
  DEFINE_FPI(istr_fisttp,fpiTypeLInt),          /* FISTTP             [29] */
  DEFINE_FPI(istr_fst,fpiTypeDouble),           /* FST    (QWord Ptr) [2A] */
  DEFINE_FPI(istr_fstp,fpiTypeDouble),          /* FSTP   (QWord Ptr) [2B] */
  DEFINE_FPI(istr_frstor,fpiTypeStatus),        /* FRSTOR             [2C] */
  DEFINE_FPI(-1,0),                             /* ESC 2Dh            [2D] */
  DEFINE_FPI(istr_fnsave,fpiTypeStatus),        /* FSAVE              [2E] */
  DEFINE_FPI(istr_fnstsw,fpiTypeWord),          /* FSTSW              [2F] */

  DEFINE_FPI(istr_fiadd,fpiTypeWInt),           /* FIADD  (Word Ptr)  [30] */
  DEFINE_FPI(istr_fimul,fpiTypeWInt),           /* FIMUL  (Word Ptr)  [31] */
  DEFINE_FPI(istr_ficom,fpiTypeWInt),           /* FICOM  (Word Ptr)  [32] */
  DEFINE_FPI(istr_ficomp,fpiTypeWInt),          /* FICOMP (Word Ptr)  [33] */
  DEFINE_FPI(istr_fisub,fpiTypeWInt),           /* FISUB  (Word Ptr)  [34] */
  DEFINE_FPI(istr_fisubr,fpiTypeWInt),          /* FISUBR (Word Ptr)  [35] */
  DEFINE_FPI(istr_fidiv,fpiTypeWInt),           /* FIDIV  (Word Ptr)  [36] */
  DEFINE_FPI(istr_fidivr,fpiTypeWInt),          /* FIDIVR (Word Ptr)  [37] */

  DEFINE_FPI(istr_fild,fpiTypeWInt),            /* FILD   (Word Ptr)  [38] */
  DEFINE_FPI(istr_fisttp,fpiTypeWInt),          /* FISTTP             [39] */
  DEFINE_FPI(istr_fist,fpiTypeWInt),            /* FIST   (Word Ptr)  [3A] */
  DEFINE_FPI(istr_fistp,fpiTypeWInt),           /* FISTP  (Word Ptr)  [3B] */
  DEFINE_FPI(istr_fbld,fpiTypeBCD),             /* FBLD   (TByte Ptr) [3C] */
  DEFINE_FPI(istr_fild,fpiTypeLInt),            /* FILD   (QWord Ptr) [3D] */
  DEFINE_FPI(istr_fbstp,fpiTypeBCD),            /* FBSTP  (TWord Ptr) [3E] */
  DEFINE_FPI(istr_fistp,fpiTypeLInt)            /* FISTP  (QWord Ptr) [3F] */
};
COMPILE_CHECK(escInstrLenCheck,
  sizeof(esc_inst) == (sizeof(esc_inst[0])*64)
);

enum EscSpecial
{
  escS_d9d0, escS_d9e0, escS_d9e8, escS_d9f0,
  escS_d9f8, escS_dae8, escS_dbe0, escS_ded8,
  escS_dfe0
};

const int esc_inst_mod3[] = {
  DEFINE_FPIM3(istr_fadd,fpiParamStStN),    /* FADD  (DWord Ptr)  [D8] [C0] */
  DEFINE_FPIM3(istr_fmul,fpiParamStStN),    /* FMUL  (DWord Ptr)  [D8] [C8] */
  DEFINE_FPIM3(istr_fcom,fpiParamStStN),    /* FCOM  (DWord Ptr)  [D8] [D0] */
  DEFINE_FPIM3(istr_fcomp,fpiParamStStN),   /* FCOMP (DWord Ptr)  [D8] [D8] */
  DEFINE_FPIM3(istr_fsub,fpiParamStStN),    /* FSUB  (DWord Ptr)  [D8] [E0] */
  DEFINE_FPIM3(istr_fsubr,fpiParamStStN),   /* FSUBR (DWord Ptr)  [D8] [E8] */
  DEFINE_FPIM3(istr_fdiv,fpiParamStStN),    /* FDIV  (DWord Ptr)  [D8] [F0] */
  DEFINE_FPIM3(istr_fdivr,fpiParamStStN),   /* FDIVR (DWord Ptr)  [D8] [F8] */

  DEFINE_FPIM3(istr_fld,fpiParamStN),       /* FLD  (DWord Ptr)   [D9] [C0] */
  DEFINE_FPIM3(istr_fxch,fpiParamStN),      /* FXCH               [D9] [C8] */
  DEFINE_FPIM3(escS_d9d0,fpiParamNone),     /*                    [D9] [D0] */
  DEFINE_FPIM3(istr_fstp,fpiParamStN),      /*                    [D9] [D8] */
  DEFINE_FPIM3(escS_d9e0,fpiParamNone),     /*                    [D9] [E0] */
  DEFINE_FPIM3(escS_d9e8,fpiParamNone),     /*                    [D9] [E8] */
  DEFINE_FPIM3(escS_d9f0,fpiParamNone),     /*                    [D9] [F0] */
  DEFINE_FPIM3(escS_d9f8,fpiParamNone),     /*                    [D9] [F8] */

  DEFINE_FPIM3(istr_fcmovb,fpiParamStStN),  /*                    [DA] [C0] */
  DEFINE_FPIM3(istr_fcmove,fpiParamStStN),  /*                    [DA] [C8] */
  DEFINE_FPIM3(istr_fcmovbe,fpiParamStStN), /*                    [DA] [D0] */
  DEFINE_FPIM3(istr_fcmovu,fpiParamStStN),  /*                    [DA] [D8] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DA] [E0] */
  DEFINE_FPIM3(escS_dae8,fpiParamNone),     /*                    [DA] [E8] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DA] [F0] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DA] [F8] */

  DEFINE_FPIM3(istr_fcmovnb,fpiParamStStN), /*                    [DB] [C0] */
  DEFINE_FPIM3(istr_fcmovne,fpiParamStStN), /*                    [DB] [C8] */
  DEFINE_FPIM3(istr_fcmovnbe,fpiParamStStN),/*                    [DB] [D0] */
  DEFINE_FPIM3(istr_fcmovnu,fpiParamStStN), /*                    [DB] [D8] */
  DEFINE_FPIM3(escS_dbe0,fpiParamNone),     /*                    [DB] [E0] */
  DEFINE_FPIM3(istr_fucomi,fpiParamStStN),  /*                    [DB] [E8] */
  DEFINE_FPIM3(istr_fcomi,fpiParamStStN),   /*                    [DB] [F0] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DB] [F8] */

  DEFINE_FPIM3(istr_fadd,fpiParamStNSt),    /* FADD (QWord Ptr)   [DC] [C0] */
  DEFINE_FPIM3(istr_fmul,fpiParamStNSt),    /* FMUL (QWord Ptr)   [DC] [C8] */
  DEFINE_FPIM3(istr_fcom,fpiParamStN),      /*                    [DC] [D0] */
  DEFINE_FPIM3(istr_fcomp,fpiParamStN),     /*                    [DC] [D8] */
  DEFINE_FPIM3(istr_fsubr,fpiParamStNSt),   /* FSUBR(QWord Ptr)   [DC] [E0] */
  DEFINE_FPIM3(istr_fsub,fpiParamStNSt),    /* FSUB  (QWord Ptr)  [DC] [E8] */
  DEFINE_FPIM3(istr_fdivr,fpiParamStNSt),   /* FDIVR(QWord Ptr)   [DC] [F0] */
  DEFINE_FPIM3(istr_fdiv,fpiParamStNSt),    /* FDIV  (QWord Ptr)  [DC] [F8] */

  DEFINE_FPIM3(istr_ffree,fpiParamStN),     /* FLD                [DD] [C0] */
  DEFINE_FPIM3(istr_fxch4,fpiParamStN),     /* FXCH4              [DD] [C8] */
  DEFINE_FPIM3(istr_fst,fpiParamStN),       /* FST                [DD] [D0] */
  DEFINE_FPIM3(istr_fstp,fpiParamStN),      /* FSTP               [DD] [D8] */
  DEFINE_FPIM3(istr_fucom,fpiParamStNSt),   /*                    [DD] [E0] */
  DEFINE_FPIM3(istr_fucomp,fpiParamStN),    /*                    [DD] [E8] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DD] [F0] */
  DEFINE_FPIM3(-1,fpiParamNone),            /*                    [DD] [F8] */

  DEFINE_FPIM3(istr_faddp,fpiParamStNSt),   /* FADDP              [DE] [C0] */
  DEFINE_FPIM3(istr_fmulp,fpiParamStNSt),   /* FMULP              [DE] [C8] */
  DEFINE_FPIM3(istr_fcomp5,fpiParamStN),    /* FCOMP5             [DE] [D0] */
  DEFINE_FPIM3(escS_ded8,fpiParamNone),     /*                    [DE] [D8] */
  DEFINE_FPIM3(istr_fsubrp,fpiParamStNSt),  /* FSUBRP             [DE] [E0] */
  DEFINE_FPIM3(istr_fsubp,fpiParamStNSt),   /* FSUBP              [DE] [E8] */
  DEFINE_FPIM3(istr_fdivrp,fpiParamStNSt),  /* FDIVRP             [DE] [F0] */
  DEFINE_FPIM3(istr_fdivp,fpiParamStNSt),   /* FDIVP              [DE] [F8] */

  DEFINE_FPIM3(istr_ffreep,fpiParamStN),    /* FFREEP             [DF] [C0] */
  DEFINE_FPIM3(istr_fxch7,fpiParamStN),     /* FXCH7              [DF] [C8] */
  DEFINE_FPIM3(istr_fstp8,fpiParamStN),     /* FSTP8              [DF] [D0] */
  DEFINE_FPIM3(istr_fstp9,fpiParamStN),     /* FSTP9              [DF] [D8] */
  DEFINE_FPIM3(escS_dfe0,fpiParamNone),     /*                    [DF] [E0] */
  DEFINE_FPIM3(istr_fucomip,fpiParamStStN), /*                    [DF] [E8] */
  DEFINE_FPIM3(istr_fcomip,fpiParamStStN),  /*                    [DF] [F0] */
  DEFINE_FPIM3(-1,fpiParamNone)             /*                    [DF] [F8] */
};

#define ESCSPECIAL(index,i1,i2,i3,i4,i5,i6,i7,i8) \
  { istr_##i1, istr_##i2, istr_##i3, istr_##i4, \
    istr_##i5, istr_##i6, istr_##i7, istr_##i8 }
const int esc_inst_special[][8] =
{
  ESCSPECIAL(d9d0,fnop, _,       _,     _,      _,      _,     _,      _),
  ESCSPECIAL(d9e0,fchs, fabs,    _,     _,      ftst,   fxam,  _,      _),
  ESCSPECIAL(d9e8,fldl, fldl2t,  fldl2e,fldpi,  fldlg2, fldln2,fldz,   _),
  ESCSPECIAL(d9f0,f2xm1,fyl2x,   fptan, fpatan, fxtract,fpreml,fdecstp,fincstp),
  ESCSPECIAL(d9f8,fprem,fyl2xpl, fsqrt, fsincos,frndint,fscale,fsin,   fcos),
  ESCSPECIAL(dae8,_,    fucompp, _,     _,      _,      _,     _,      _),
  ESCSPECIAL(dbe0,feni, fdisi,   fclex, finit,  fsetpm, frstpm,_,      _),
  ESCSPECIAL(ded8,_,    fcompp,  _,     _,      _,      _,     _,      _),
  ESCSPECIAL(dfe0,fstsw,fstdw,   fstsg, _,      _,      _,     _,      _)
};

const INST_3DNOW  k6_3DNow_instr[] = {
  {    -1        },  // [00]
  {    -1        },  // [01]
  {    -1        },  // [02]
  {    -1        },  // [03]
  {    -1        },  // [04]
  {    -1        },  // [05]
  {    -1        },  // [06]
  {    -1        },  // [07]
  {    -1        },  // [08]
  {    -1        },  // [09]
  {    -1        },  // [0A]
  {    -1        },  // [0B]
  { istr_pi2fw   },  // [0C]
  { istr_pi2fd   },  // [0D]
  {    -1        },  // [0E]
  {    -1        },  // [0F]
  {    -1        },  // [10]
  {    -1        },  // [11]
  {    -1        },  // [12]
  {    -1        },  // [13]
  {    -1        },  // [14]
  {    -1        },  // [15]
  {    -1        },  // [16]
  {    -1        },  // [17]
  {    -1        },  // [18]
  {    -1        },  // [19]
  {    -1        },  // [1A]
  {    -1        },  // [1B]
  { istr_pf2iw   },  // [1C]
  { istr_pf2id   },  // [1D]
  {    -1        },  // [1E]
  {    -1        },  // [1F]
  {    -1        },  // [20]
  {    -1        },  // [21]
  {    -1        },  // [22]
  {    -1        },  // [23]
  {    -1        },  // [24]
  {    -1        },  // [25]
  {    -1        },  // [26]
  {    -1        },  // [27]
  {    -1        },  // [28]
  {    -1        },  // [29]
  {    -1        },  // [2A]
  {    -1        },  // [2B]
  {    -1        },  // [2C]
  {    -1        },  // [2D]
  {    -1        },  // [2E]
  {    -1        },  // [2F]
  {    -1        },  // [30]
  {    -1        },  // [31]
  {    -1        },  // [32]
  {    -1        },  // [33]
  {    -1        },  // [34]
  {    -1        },  // [35]
  {    -1        },  // [36]
  {    -1        },  // [37]
  {    -1        },  // [38]
  {    -1        },  // [39]
  {    -1        },  // [3A]
  {    -1        },  // [3B]
  {    -1        },  // [3C]
  {    -1        },  // [3D]
  {    -1        },  // [3E]
  {    -1        },  // [3F]
  {    -1        },  // [40]
  {    -1        },  // [41]
  {    -1        },  // [42]
  {    -1        },  // [43]
  {    -1        },  // [44]
  {    -1        },  // [45]
  {    -1        },  // [46]
  {    -1        },  // [47]
  {    -1        },  // [48]
  {    -1        },  // [49]
  {    -1        },  // [4A]
  {    -1        },  // [4B]
  {    -1        },  // [4C]
  {    -1        },  // [4D]
  {    -1        },  // [4E]
  {    -1        },  // [4F]
  {    -1        },  // [50]
  {    -1        },  // [51]
  {    -1        },  // [52]
  {    -1        },  // [53]
  {    -1        },  // [54]
  {    -1        },  // [55]
  {    -1        },  // [56]
  {    -1        },  // [57]
  {    -1        },  // [58]
  {    -1        },  // [59]
  {    -1        },  // [5A]
  {    -1        },  // [5B]
  {    -1        },  // [5C]
  {    -1        },  // [5D]
  {    -1        },  // [5E]
  {    -1        },  // [5F]
  {    -1        },  // [60]
  {    -1        },  // [61]
  {    -1        },  // [62]
  {    -1        },  // [63]
  {    -1        },  // [64]
  {    -1        },  // [65]
  {    -1        },  // [66]
  {    -1        },  // [67]
  {    -1        },  // [68]
  {    -1        },  // [69]
  {    -1        },  // [6A]
  {    -1        },  // [6B]
  {    -1        },  // [6C]
  {    -1        },  // [6D]
  {    -1        },  // [6E]
  {    -1        },  // [6F]
  {    -1        },  // [70]
  {    -1        },  // [71]
  {    -1        },  // [72]
  {    -1        },  // [73]
  {    -1        },  // [74]
  {    -1        },  // [75]
  {    -1        },  // [76]
  {    -1        },  // [77]
  {    -1        },  // [78]
  {    -1        },  // [79]
  {    -1        },  // [7A]
  {    -1        },  // [7B]
  {    -1        },  // [7C]
  {    -1        },  // [7D]
  {    -1        },  // [7E]
  {    -1        },  // [7F]
  {    -1        },  // [80]
  {    -1        },  // [81]
  {    -1        },  // [82]
  {    -1        },  // [83]
  {    -1        },  // [84]
  {    -1        },  // [85]
  {    -1        },  // [86]
  {    -1        },  // [87]
  {    -1        },  // [88]
  {    -1        },  // [89]
  { istr_pfnacc  },  // [8A]
  {    -1        },  // [8B]
  {    -1        },  // [8C]
  {    -1        },  // [8D]
  { istr_pfpnacc },  // [8E]
  {    -1        },  // [8F]
  { istr_pfcmpge },  // [90]
  {    -1        },  // [91]
  {    -1        },  // [92]
  {    -1        },  // [93]
  { istr_pfmin   },  // [94]
  {    -1        },  // [95]
  { istr_pfrcp   },  // [96]
  { istr_pfrsqrt },  // [97]
  {    -1        },  // [98]
  {    -1        },  // [99]
  { istr_pfsub   },  // [9A]
  {    -1        },  // [9B]
  {    -1        },  // [9C]
  {    -1        },  // [9D]
  { istr_pfadd   },  // [9E]
  {    -1        },  // [9F]
  { istr_pfcmpgt },  // [A0]
  {    -1        },  // [A1]
  {    -1        },  // [A2]
  {    -1        },  // [A3]
  { istr_pfmax   },  // [A4]
  {    -1        },  // [A5]
  { istr_pfrcpit1},  // [A6]
  { istr_pfrsqit1},  // [A7]
  {    -1        },  // [A8]
  {    -1        },  // [A9]
  { istr_pfsubr  },  // [AA]
  {    -1        },  // [AB]
  {    -1        },  // [AC]
  {    -1        },  // [AD]
  { istr_pfacc   },  // [AE]
  {    -1        },  // [AF]
  { istr_pfcmpeq },  // [B0]
  {    -1        },  // [B1]
  {    -1        },  // [B2]
  {    -1        },  // [B3]
  { istr_pfmul   },  // [B4]
  {    -1        },  // [B5]
  { istr_pfrcpit2},  // [B6]
  { istr_pmulhrw },  // [B7]
  {    -1        },  // [B8]
  {    -1        },  // [B9]
  {    -1        },  // [BA]
  { istr_pswapd  },  // [BB]
  {    -1        },  // [BC]
  {    -1        },  // [BD]
  {    -1        },  // [BE]
  { istr_pavgusb },  // [BF]
  {    -1        },  // [C0]
  {    -1        },  // [C1]
  {    -1        },  // [C2]
  {    -1        },  // [C3]
  {    -1        },  // [C4]
  {    -1        },  // [C5]
  {    -1        },  // [C6]
  {    -1        },  // [C7]
  {    -1        },  // [C8]
  {    -1        },  // [C9]
  {    -1        },  // [CA]
  {    -1        },  // [CB]
  {    -1        },  // [CC]
  {    -1        },  // [CD]
  {    -1        },  // [CE]
  {    -1        },  // [CF]
  {    -1        },  // [D0]
  {    -1        },  // [D1]
  {    -1        },  // [D2]
  {    -1        },  // [D3]
  {    -1        },  // [D4]
  {    -1        },  // [D5]
  {    -1        },  // [D6]
  {    -1        },  // [D7]
  {    -1        },  // [D8]
  {    -1        },  // [D9]
  {    -1        },  // [DA]
  {    -1        },  // [DB]
  {    -1        },  // [DC]
  {    -1        },  // [DD]
  {    -1        },  // [DE]
  {    -1        },  // [DF]
  {    -1        },  // [E0]
  {    -1        },  // [E1]
  {    -1        },  // [E2]
  {    -1        },  // [E3]
  {    -1        },  // [E4]
  {    -1        },  // [E5]
  {    -1        },  // [E6]
  {    -1        },  // [E7]
  {    -1        },  // [E8]
  {    -1        },  // [E9]
  {    -1        },  // [EA]
  {    -1        },  // [EB]
  {    -1        },  // [EC]
  {    -1        },  // [ED]
  {    -1        },  // [EE]
  {    -1        },  // [EF]
  {    -1        },  // [F0]
  {    -1        },  // [F1]
  {    -1        },  // [F2]
  {    -1        },  // [F3]
  {    -1        },  // [F4]
  {    -1        },  // [F5]
  {    -1        },  // [F6]
  {    -1        },  // [F7]
  {    -1        },  // [F8]
  {    -1        },  // [F9]
  {    -1        },  // [FA]
  {    -1        },  // [FB]
  {    -1        },  // [FC]
  {    -1        },  // [FD]
  {    -1        },  // [FE]
  {    -1        }   // [FF]
};

COMPILE_CHECK(k6_3dInstrLenCheck,
  sizeof(k6_3DNow_instr) == (sizeof(k6_3DNow_instr[0])*256)
);

// !!! test processor for SSE2
const INST_F3XMM f3Xmm_instr[] = {
  { 0, -1 },
  { modregVssWss,    istr_movss     },  // [10]
  { modregWssVss,    istr_movss     },  // [11]
  { modregVpsWps,    istr_movsldup  },  // [12] SSE2
  { modregVpsWps,    istr_movshdup  },  // [16] SSE2
  { modregVssEd,     istr_cvtsi2ss  },  // [2A]
  { modregGdWss,     istr_cvttss2si },  // [2C]
  { modregGdWss,     istr_cvtss2si  },  // [2D]
  { modregVssWss,    istr_sqrtss    },  // [51]
  { modregVssWss,    istr_rsqrtss   },  // [52]
  { modregVssWss,    istr_rcpss     },  // [53]
  { modregVssWss,    istr_addss     },  // [58]
  { modregVssWss,    istr_mulss     },  // [59]
  { modregVssWss,    istr_cvtss2sd  },  // [5A] SSE2
  { modregVdqWps,    istr_cvttps2dq },  // [5B] SSE2
  { modregVssWss,    istr_subss     },  // [5C]
  { modregVssWss,    istr_minss     },  // [5D]
  { modregVssWss,    istr_divss     },  // [5E]
  { modregVssWss,    istr_maxss     },  // [5F]
  { modregVdqWdq,    istr_movdqu    },  // [6F] SSE2
  { modregVpsWpsIub, istr_pshufhw   },  // [70] SSE2
  { modregVqWq,      istr_movq      },  // [7E] SSE2
  { modregWdqVdq,    istr_movdqu    },  // [7F] SSE2
  { modregVssWssIub, istr_cmpss     },  // [C2]
  { modregVdqQq,     istr_movq2dq   },  // [D6] SSE2
  { modregVpdWdq,    istr_cvtdq2pd  },  // [E6] SSE2
};

enum {
  F3OFF_NULL,
  F3OFF_10,
  F3OFF_11,
  F3OFF_12,
  F3OFF_16,
  F3OFF_2A,
  F3OFF_2C,
  F3OFF_2D,
  F3OFF_51,
  F3OFF_52,
  F3OFF_53,
  F3OFF_58,
  F3OFF_59,
  F3OFF_5A,
  F3OFF_5B,
  F3OFF_5C,
  F3OFF_5D,
  F3OFF_5E,
  F3OFF_5F,
  F3OFF_6F,
  F3OFF_70,
  F3OFF_7E,
  F3OFF_7F,
  F3OFF_C2,
  F3OFF_D6,
  F3OFF_E6,
};

const INST_F3XMM f2Xmm_instr[] = {
  { 0, -1 },
  { modregVsdWsd,    istr_movsd     },  // [10]
  { modregWsdVsd,    istr_movsd     },  // [11]
  { modregVqWq,      istr_movddup   },  // [12]
  { modregVsdEd,     istr_cvtsi2sd  },  // [2A]
  { modregGdWsd,     istr_cvttsd2si },  // [2C]
  { modregGdWsd,     istr_cvtsd2si  },  // [2D]
  { modregVsdWsd,    istr_sqrtsd    },  // [51]
  { modregVsdWsd,    istr_addsd     },  // [58]
  { modregVsdWsd,    istr_mulsd     },  // [59]
  { modregVsdWsd,    istr_cvtsd2ss  },  // [5A]
  { modregVsdWsd,    istr_subsd     },  // [5C]
  { modregVsdWsd,    istr_minsd     },  // [5D]
  { modregVsdWsd,    istr_divsd     },  // [5E]
  { modregVsdWsd,    istr_maxsd     },  // [5F]
  { modregVpsWpsIub, istr_pshuflw   },  // [70]
  { modregVpsWps,    istr_haddps    },  // [7C]
  { modregVpsWps,    istr_hsubps    },  // [7D]
  { modregVsdWsdIub, istr_cmpsd     },  // [C2]
  { modregVpsWps,    istr_addsubps  },  // [D0]
  { modregPqQq,      istr_movdq2q   },  // [D6]
  { modregVdqWpd,    istr_cvtpd2dq  },  // [E6]
  { modregVdqWdq,    istr_lddqu     },  // [F0]
};

enum {
  F2OFF_NULL,
  F2OFF_10,
  F2OFF_11,
  F2OFF_12,
  F2OFF_2A,
  F2OFF_2C,
  F2OFF_2D,
  F2OFF_51,
  F2OFF_58,
  F2OFF_59,
  F2OFF_5A,
  F2OFF_5C,
  F2OFF_5D,
  F2OFF_5E,
  F2OFF_5F,
  F2OFF_70,
  F2OFF_7C,
  F2OFF_7D,
  F2OFF_C2,
  F2OFF_D0,
  F2OFF_D6,
  F2OFF_E6,
  F2OFF_F0,
};

const INST_F3XMM p66Xmm_instr[] = {
  { 0, -1 },
  { modregVpdWpd,    istr_movupd     },  // [10]
  { modregWpdVpd,    istr_movupd     },  // [11]
  { modregVqWq,      istr_movlpd     },  // [12]
  { modregWqVq,      istr_movlpd     },  // [13]
  { modregVpdWps,    istr_unpcklpd   },  // [14]
  { modregVpdWps,    istr_unpckhpd   },  // [15]
  { modregVqWq,      istr_movhpd     },  // [16]
  { modregWqVq,      istr_movhpd     },  // [17]
  { modregVpdWpd,    istr_movapd     },  // [28]
  { modregWpdVpd,    istr_movapd     },  // [29]
  { modregVpdQq,     istr_cvtpi2pd   },  // [2A]
  { modregWpdVpd,    istr_movntpd    },  // [2B]
  { modregQdqWpd,    istr_cvttpd2pi  },  // [2C]
  { modregQdqWpd,    istr_cvtpd2pi   },  // [2D]
  { modregVsdWsd,    istr_ucomisd    },  // [2E]
  { modregVsdWsd,    istr_comisd     },  // [2F]
  { modregEdVpd,     istr_movmskpd   },  // [50]
  { modregVpdWpd,    istr_sqrtpd     },  // [51]
  { modregVpdWpd,    istr_andpd      },  // [54]
  { modregVpdWpd,    istr_andnpd     },  // [55]
  { modregVpdWpd,    istr_orpd       },  // [56]
  { modregVpdWpd,    istr_xorpd      },  // [57]
  { modregVpdWpd,    istr_addpd      },  // [58]
  { modregVpdWpd,    istr_mulpd      },  // [59]
  { modregVpsWpd,    istr_cvtpd2ps   },  // [5A]
  { modregVdqWps,    istr_cvtps2dq   },  // [5B]
  { modregVpdWpd,    istr_subpd      },  // [5C]
  { modregVpdWpd,    istr_minpd      },  // [5D]
  { modregVpdWpd,    istr_divpd      },  // [5E]
  { modregVpdWpd,    istr_maxpd      },  // [5F]
  { modregVdqWdq,    istr_punpcklbw  },  // [60]
  { modregVdqWdq,    istr_punpcklwd  },  // [61]
  { modregVdqWdq,    istr_punpckldq  },  // [62]
  { modregVdqWdq,    istr_packsswb   },  // [63]
  { modregVdqWdq,    istr_pcmpgtb    },  // [64]
  { modregVdqWdq,    istr_pcmpgtw    },  // [65]
  { modregVdqWdq,    istr_pcmpgtd    },  // [66]
  { modregVdqWdq,    istr_packuswb   },  // [67]
  { modregVdqWdq,    istr_punpcklqdq },  // [6C]
  { modregVdqWdq,    istr_punpckhqdq },  // [6D]
  { modregVdqWdq,    istr_movdqa     },  // [6F]
  { modregVpsWpsIub, istr_pshufd     },  // [70]
  { modregVpdWpd,    istr_haddpd     },  // [7C]
  { modregVpdWpd,    istr_hsubpd     },  // [7D]
  { modregWdqVdq,    istr_movdqa     },  // [7F]
  // !!! Ib should be checked, change instruction from Ib codes !!!
  { modregVpdWpdIub, istr_cmppd      },  // [C2]
  { modregVpdWpdIub, istr_shufpd     },  // [C6]
  { modregVpdWpd,    istr_addsubpd   },  // [D0]
  { modregWqVq,      istr_movq       },  // [D6]
  { modregVdqWpd,    istr_cvttpd2dq  },  // [E6]
  { modregWdqVdq,    istr_movntdq    },  // [E7]
  { modregVdqWdq,    istr_maskmovdqu },  // [F7]
};
// 60-B,6E,74-6,7E,C4-5,D1-5,D7,D8-F,E0-5,E8-EF,F1-6,F8-E XMM equivalent of MMX
// - tested in mod_reg2

enum {
  P66OFF_NULL,
  P66OFF_10,
  P66OFF_11,
  P66OFF_12,
  P66OFF_13,
  P66OFF_14,
  P66OFF_15,
  P66OFF_16,
  P66OFF_17,
  P66OFF_28,
  P66OFF_29,
  P66OFF_2A,
  P66OFF_2B,
  P66OFF_2C,
  P66OFF_2D,
  P66OFF_2E,
  P66OFF_2F,
  P66OFF_50,
  P66OFF_51,
  P66OFF_54,
  P66OFF_55,
  P66OFF_56,
  P66OFF_57,
  P66OFF_58,
  P66OFF_59,
  P66OFF_5A,
  P66OFF_5B,
  P66OFF_5C,
  P66OFF_5D,
  P66OFF_5E,
  P66OFF_5F,
  P66OFF_60,
  P66OFF_61,
  P66OFF_62,
  P66OFF_63,
  P66OFF_64,
  P66OFF_65,
  P66OFF_66,
  P66OFF_67,
  P66OFF_6C,
  P66OFF_6D,
  P66OFF_6F,
  P66OFF_70,
  P66OFF_7C,
  P66OFF_7D,
  P66OFF_7F,
  P66OFF_C2,
  P66OFF_C6,
  P66OFF_D0,
  P66OFF_D6,
  P66OFF_E6,
  P66OFF_E7,
  P66OFF_F7,
};

// do not waste memory
const uchar xmm_offsets[][3] = {
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [00]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [01]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [02]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [03]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [04]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [05]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [06]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [07]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [08]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [09]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [0F]
  { F3OFF_10  ,  F2OFF_10  ,  P66OFF_10   },  // [10]
  { F3OFF_11  ,  F2OFF_11  ,  P66OFF_11   },  // [11]
  { F3OFF_12  ,  F2OFF_12  ,  P66OFF_12   },  // [12]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_13   },  // [13]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_14   },  // [14]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_15   },  // [15]
  { F3OFF_16,    F2OFF_NULL,  P66OFF_16   },  // [16]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_17   },  // [17]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [18]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [19]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [1F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [20]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [21]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [22]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [23]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [24]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [25]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [26]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [27]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_28   },  // [28]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_29   },  // [29]
  { F3OFF_2A  ,  F2OFF_2A  ,  P66OFF_2A   },  // [2A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_2B   },  // [2B]
  { F3OFF_2C  ,  F2OFF_2C  ,  P66OFF_2C   },  // [2C]
  { F3OFF_2D  ,  F2OFF_2D  ,  P66OFF_2D   },  // [2D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_2E   },  // [2E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_2F   },  // [2F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [30]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [31]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [32]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [33]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [34]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [35]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [36]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [37]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [38]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [39]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [3F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [40]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [41]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [42]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [43]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [44]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [45]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [46]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [47]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [48]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [49]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [4F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_50   },  // [50]
  { F3OFF_51  ,  F2OFF_51  ,  P66OFF_51   },  // [51]
  { F3OFF_52  ,  F2OFF_NULL,  P66OFF_NULL },  // [52]
  { F3OFF_53  ,  F2OFF_NULL,  P66OFF_NULL },  // [53]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_54   },  // [54]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_55   },  // [55]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_56   },  // [56]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_57   },  // [57]
  { F3OFF_58  ,  F2OFF_58  ,  P66OFF_58   },  // [58]
  { F3OFF_59  ,  F2OFF_59  ,  P66OFF_59   },  // [59]
  { F3OFF_5A  ,  F2OFF_5A  ,  P66OFF_5A   },  // [5A]
  { F3OFF_5B  ,  F2OFF_NULL,  P66OFF_5B   },  // [5B]
  { F3OFF_5C  ,  F2OFF_5C  ,  P66OFF_5C   },  // [5C]
  { F3OFF_5D  ,  F2OFF_5D  ,  P66OFF_5D   },  // [5D]
  { F3OFF_5E  ,  F2OFF_5E  ,  P66OFF_5E   },  // [5E]
  { F3OFF_5F  ,  F2OFF_5F  ,  P66OFF_5F   },  // [5F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_60   },  // [60]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_61   },  // [61]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_62   },  // [62]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_63   },  // [63]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_64   },  // [64]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_65   },  // [65]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_66   },  // [66]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_67   },  // [67]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [68]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [69]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [6A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [6B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_6C   },  // [6C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_6D   },  // [6D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [6E]
  { F3OFF_6F  ,  F2OFF_NULL,  P66OFF_6F   },  // [6F]
  { F3OFF_70  ,  F2OFF_NULL,  P66OFF_NULL },  // [70]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [71]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [72]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [73]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [74]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [75]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [76]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [77]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [78]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [79]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [7A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [7B]
  { F3OFF_NULL,  F2OFF_7C  ,  P66OFF_7C   },  // [7C]
  { F3OFF_NULL,  F2OFF_7D  ,  P66OFF_7D   },  // [7D]
  { F3OFF_7E  ,  F2OFF_NULL,  P66OFF_NULL },  // [7E]
  { F3OFF_7F  ,  F2OFF_NULL,  P66OFF_7F   },  // [7F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [80]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [81]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [82]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [83]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [84]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [85]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [86]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [87]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [88]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [89]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [8F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [90]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [91]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [92]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [93]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [94]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [95]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [96]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [97]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [98]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [99]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9A]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9B]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9C]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9D]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9E]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [9F]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A1]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A5]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [A9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AD]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [AF]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B1]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B5]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [B9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BD]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [BF]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C1]
  { F3OFF_C2  ,  F2OFF_C2  ,  P66OFF_C2   },  // [C2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C5]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_C6   },  // [C6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [C9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CD]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [CF]
  { F3OFF_NULL,  F2OFF_D0  ,  P66OFF_D0   },  // [D0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D1]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D5]
  { F3OFF_D6  ,  F2OFF_D6  ,  P66OFF_D6   },  // [D6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [D9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DD]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [DF]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E1]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E5]
  { F3OFF_E6  ,  F2OFF_E6  ,  P66OFF_E6   },  // [E6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_E7   },  // [E7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [E9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [EA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [EB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [EC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [ED]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [EE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [EF]
  { F3OFF_NULL,  F2OFF_F0  ,  P66OFF_NULL },  // [F0]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F1]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F2]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F3]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F4]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F5]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F6]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_F7   },  // [F7]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F8]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [F9]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FA]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FB]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FC]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FD]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FE]
  { F3OFF_NULL,  F2OFF_NULL,  P66OFF_NULL },  // [FF]
};

COMPILE_CHECK(xmmLenCheck,
  sizeof(xmm_offsets) == (sizeof(xmm_offsets[0])*256)
);
