/*
PeRdr - PE file disassembler
Copyright (C) 1999-2006 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "x86instr.h"

struct inst_3DNow {
    int     instruction;
};
typedef struct inst_3DNow INST_3DNOW;

struct inst_f3Xmm {
    int     special;
    int     instruction;
};
typedef struct inst_f3Xmm INST_F3XMM;

enum Regs 
{
  regNone = 0,
  
  regAL,
  regCL,
  regDL,
  regBL,
  regAH,
  regCH,
  regDH,
  regBH,

  regAX,   // Or EAX, etc. as may arise
  regCX,
  regDX,
  regBX,
  regSP,
  regBP,
  regSI,
  regDI,

  regES,   // Segment register identifiers
  regCS,
  regSS,
  regDS,
  regFS,
  regGS
};


enum Modreg
{
// flag per MOD_REG operands
// bits 2-0 size of registry or memory (if not specified)
  modregSize           =7,
  modregByte           =0,
  modregVar            =1,
  modregDword          =2,
  modregMmx            =3,
// !!! warning: conflict with some memory type, not use now !!!
  modregXmm            =4,

// bits 7-3 puntatori vari e tipo (solo per mem)
  modregMemStartBit    =3,
  modregMemType        =31<<modregMemStartBit,
  modregMemByte        =1<<modregMemStartBit,
  modregMemWord        =2<<modregMemStartBit,
  modregMemDword       =3<<modregMemStartBit,
  modregMemQword       =4<<modregMemStartBit,
  modregMemPacketDword =5<<modregMemStartBit, // MMX or 32bit memory
  modregMemPointer     =6<<modregMemStartBit,
  modregMemBound       =7<<modregMemStartBit,
  modregMemDescriptor  =8<<modregMemStartBit,
  modregMemNone        =9<<modregMemStartBit,  // only a pointer (LEA & INVLPG)
  modregMemPacket      =10<<modregMemStartBit,
  modregMemSPacket     =11<<modregMemStartBit, // XMM or 32bit float
  modregMem2SPacket    =12<<modregMemStartBit,
  modregMem4SPacket    =13<<modregMemStartBit,

// bits 9-8 mem/reg, reg/mem or mem only
  modregTypeStartBit   =8,
  modregType           =3<<modregTypeStartBit,
  modregMemReg         =0<<modregTypeStartBit,
  modregRegMem         =1<<modregTypeStartBit,
  modregMem            =2<<modregTypeStartBit,
  modregReg            =3<<modregTypeStartBit,

// bits 12-10 third operand
  modregThirdStartBit  =10,
  modregThird          =7<<modregThirdStartBit,
  modregIsb            =1<<modregThirdStartBit,
  modregIub            =2<<modregThirdStartBit,
  modregIv             =3<<modregThirdStartBit,
  modreg1              =4<<modregThirdStartBit,
  modregCL             =5<<modregThirdStartBit,

// Manca Mp
// per CR, DR e TR altra funzione
// per EwsW SwEw altra funzione

// gruppi: funzione gruppo in instr passo numero gruppo, special il tipo
// se non ha tipo passo -indice ad un vettore per i tipi
// sdoppio mod_reg per passare il primo byte

  modregEbGb    =modregByte|modregMemReg,
  modregEvGv    =modregVar |modregMemReg,
  modregGbEb    =modregByte|modregRegMem,
  modregGvEv    =modregVar |modregRegMem,

// segment
  modregEwSw    =modregVar |modregMemReg|modregMemWord,
  modregSwEw    =modregVar |modregRegMem|modregMemWord,

// arpl (this is actually EwGv, but who uses ARPL anyway?)
  modregEwGw    =modregVar |modregMemReg|modregMemWord,

  modregEv      =modregVar |modregMem,
  modregEb1     =modregByte|modregMem|modreg1,
  modregEv1     =modregVar |modregMem|modreg1,
  modregEbCL    =modregByte|modregMem|modregCL,
  modregEvCL    =modregVar |modregMem|modregCL,
  modregGvMa    =modregVar |modregRegMem|modregMemBound,
  modregGvMp    =modregVar |modregRegMem|modregMemPointer,
  modregEbIsb   =modregByte|modregMem|modregIsb,
  modregEbIub   =modregByte|modregMem|modregIub,
  modregEvIv    =modregVar |modregMem|modregIv,
  modregGvEvIv  =modregVar |modregRegMem|modregIv,
  modregGvEvIsb =modregVar |modregRegMem|modregIsb,
  modregGvEvIub =modregVar |modregRegMem|modregIub,
  modregEb      =modregByte|modregMem,
  modregGvEb    =modregVar |modregRegMem|modregMemByte,
  modregGvEw    =modregVar |modregRegMem|modregMemWord,
  modregEvGvIsb =modregVar |modregMemReg|modregIsb,
  modregEvGvIub =modregVar |modregMemReg|modregIub,
  modregEvGvCL  =modregVar |modregMemReg|modregCL,
  modregEvIsb   =modregVar |modregMem|modregIsb,
  modregEvIub   =modregVar |modregMem|modregIub,

// Memory only
  modregEp      =modregMem|modregMemPointer,
  modregEw      =modregMem|modregMemWord,
  modregMs      =modregMem|modregMemDescriptor,
  modregMn      =modregMem|modregMemNone,
  modregMq      =modregMem|modregMemQword,
// LEA
  modregGvMn    =modregVar |modregRegMem|modregMemNone,

// Mmx
  modregPqQd    =modregRegMem|modregMmx|modregMemPacketDword,
  modregPqQq    =modregRegMem|modregMmx|modregMemQword,
  modregPqQqIub =modregPqQq|modregIub,
  modregQqPq    =modregMemReg|modregMmx|modregMemQword,
  modregPqIsb   =modregReg|modregMmx|modregIsb,
  modregPqIub   =modregReg|modregMmx|modregIub,
  modregPdEd    =modregRegMem|modregMmx|modregMemDword,
  modregEdPd    =modregMemReg|modregMmx|modregMemDword,
  modregPqEdIub =modregPdEd|modregIub,
  modregPqEwIub =modregRegMem|modregMmx|modregMemWord|modregIub,
  modregGdQq    =modregRegMem|modregDword|modregMemPacket,
  modregGdQqIub =modregGdQq|modregIub,

// Xmm
// difference from ps and ss raise from opcode
  modregWpsVps     =modregMemReg|modregXmm,
  modregVpsWps     =modregRegMem|modregXmm,
  modregVpsWpsIub  =modregRegMem|modregXmm|modregIub,
  modregVpsQq      =modregRegMem|modregXmm|modregMemPacket,
  modregPqWq       =modregRegMem|modregMmx|modregMem2SPacket,
  modregVqWq       =modregRegMem|modregXmm|modregMem2SPacket,
  modregWqVq       =modregMemReg|modregXmm|modregMem2SPacket,
  modregVssWss     =modregRegMem|modregXmm|modregMemSPacket,
  modregVssWssIub  =modregRegMem|modregXmm|modregMemSPacket|modregIub,
  modregVssEd      =modregRegMem|modregXmm|modregMemDword,
  modregGdWss      =modregRegMem|modregDword|modregMem4SPacket,
  modregWssVss     =modregMemReg|modregXmm|modregMemSPacket,
  modregEdVps      =modregMemReg|modregXmm|modregMemDword,
  modregMdGd       =modregMemReg|modregDword|modregMemDword,

// !!! do this properly
  modregEdVpd      =modregMemReg|modregXmm|modregMemDword,
  modregGdWsd      =modregRegMem|modregDword|modregMem4SPacket,
  modregVsdEd      =modregRegMem|modregXmm|modregMemDword,
  modregVpdWdq     =modregRegMem|modregXmm,
  modregWpdVpd     =modregMemReg|modregXmm,
  modregVpdWpd     =modregRegMem|modregXmm,
  modregVpdWpdIub  =modregRegMem|modregXmm|modregIub,
  modregVpdWps     =modregRegMem|modregXmm,
  modregWdqVdq     =modregMemReg|modregXmm,
  modregVdqWdq     =modregRegMem|modregXmm,
  modregQdqWpd     =modregRegMem|modregXmm,
  modregVdqWpd     =modregRegMem|modregXmm,
  modregVdqWps     =modregRegMem|modregXmm,
  modregVdqQq      =modregRegMem|modregXmm,
  modregVpdQq      =modregRegMem|modregXmm,
  modregVpsWpd     =modregRegMem|modregXmm,
  modregVpsWdq     =modregRegMem|modregXmm,
  modregWsdVsd     =modregMemReg|modregXmm,
  modregVsdWsd     =modregRegMem|modregXmm,
  modregVsdWsdIub  =modregRegMem|modregXmm|modregIub,
};

// special
enum Movcregs
{
  movcregR32CRx =0,
  movcregCRxr32 =1,
  movcregR32DRx =2,
  movcregDRxr32 =3,
  movcregR32TRx =4,
  movcregTRxr32 =5
};

extern const INST_3DNOW  k6_3DNow_instr[];
extern const INST_F3XMM  f3Xmm_instr[];
extern const INST_F3XMM  f2Xmm_instr[];
extern const INST_F3XMM  p66Xmm_instr[];
extern const uchar       xmm_offsets[][3];
#define F3XMM(n)   f3Xmm_instr[xmm_offsets[n][0]]
#define F2XMM(n)   f2Xmm_instr[xmm_offsets[n][1]]
#define P66XMM(n) p66Xmm_instr[xmm_offsets[n][2]]

#define DEFINE_FPI(istr,type) ((istr<<8)|type)
#define FPI_ISTR(i) (i>>8)
#define FPI_TYPE(i) (i&255)
enum FpiTypes
{
  fpiTypeWInt=0,fpiTypeSInt,fpiTypeLInt,                  // interi
  fpiTypeSingle,fpiTypeDouble,fpiTypeExtended,fpiTypeBCD, // float
  fpiTypeWord,fpiTypeEnv,fpiTypeStatus,                   // altri
  fpiTypeCount,
  fpiTypeImplicitMemory = fpiTypeEnv
};
extern const int esc_inst[];

#define DEFINE_FPIM3(istr,param) DEFINE_FPI(istr,param)
#define FPIM3_ISTR(i) FPI_ISTR(i)
#define FPIM3_PARAM(i) FPI_TYPE(i)
enum FpiParams
{
  fpiParamNone,fpiParamStNSt,fpiParamStN,fpiParamStStN
};
extern const int esc_inst_mod3[];
extern const int esc_inst_special[][8];


